# Cheat sheet for ROS2 CLI (very useful for beginners as well as experienced players!)
[ROS2 CLI Cheat Sheet](https://github.com/ubuntu-robotics/ros2_cheats_sheet/blob/master/cli/cli_cheats_sheet.pdf)

# Installation instructions of ROS2 container for LiU-Home-Wreckers
This guide will walk you through how to install a docker container for running this project in.
Since this process is done almost entirely using a docker-container, it will be pretty easy.
This guide will assume you already have an understanding of what docker and docker-compose is. If you run into trouble while installing, feel free to ask for help!

In its' current form, the  Dockerfiles will install:
- ROS 2 Foxy Fitzroy - as the main framework for the project.
- Python 3 - for runnin' da code.
- colcon - for builin' da code.
- libqi - an open source C++ framework for interacting with the aldebaran robots.
- CUDA version 10.2, CuDnn 7.6.5 - for doing fast computations on the GPU.
- PyTorch - Machine learning library (it should also be possible to install TensorFlow in this container if needed).
- Utilites: nano vim emacs ping git sed htop zsh

As you can see, these docker containers are made to compile on systems with one or more Nvidia GPUs.
If you don't have one of these, make sure to ALWAYS use `NO_GPU=1` when building.

This guide is aimed at Linux Debian distros, but the process should be similar on other platforms.

## 1.0 Getting the tools
Install docker.io and nvidia-docker:\
`sudo apt install docker.io`

Give docker permission to access the screen:\
`xhost +local:docker`

Install Docker Compose:\
`sudo curl -L "https://github.com/docker/compose/releases/download/1.27.4/docker-compose-$(uname -s)-$(uname -m)" -o /usr/local/bin/docker-compose`

And then to allow running: \
`sudo chmod +x /usr/local/bin/docker-compose`

<hr>

If you have an **Nvidia GPU** you should probably use them in the container. Run the following command too:\
`sudo apt install nvidia-docker2`

If you do this, make sure to also set docker's default runtime to nvidia by opening */etc/docker/daemon.json*:

>{\
>&ensp;"runtimes": {\
>&ensp;&ensp;&ensp;"nvidia": {\
>&ensp;&ensp;&ensp;&ensp;&ensp;"path": "nvidia-container-runtime",\
>&ensp;&ensp;&ensp;&ensp;&ensp;"runtimeArgs": []\
>&ensp;&ensp;}\
>&ensp;},\
>&ensp;"default-runtime": "nvidia"\
>}


Go ahead and restart the docker daemon too:\
`sudo service docker restart`


## 1.1 Getting the repository
Clone the repo:\
`git clone git@gitlab.liu.se:liuhomewreckers/liu-home-wreckers.git`

You will need to setup ssh-keys with GitLab in order to push code later.

## 1.2 Building
If you want to install the whole system, just write: \
`docker-compose build` (add `--build-arg NO_GPU=1` if you don't have an Nvidia GPU)

Make sure to build the base image 'lhw' at least once to avoid any errors related to the base image not existing in your local docker library.

A huge advantage of having multiple docker containers is that you can only install for the package(s) you need: \
Example: `docker-compose build lhw_qi`

## 1.3  Workflow

### Enter specific container
Often you will only want to work with one package at a time. It is then a good idea to 
enter the container (in this case lhw_qi):\
`docker-compose run lhw_qi zsh`

You are now inside of a docker container, with this package mounted in `src`. That means that
any changes you make there will sync with the outside of the container (the host).
Any changes made in the container but outside this folder will not persist when the container is
exited (*Ctrl-D*).

**Git does not work well inside the container. Only do things in git outside it.**

To use the project, you will first have to build it:\
`colcon build --symlink-install`

If you are working with C++, you will have to rebuild with colcon everytime you want to test your code.

To access the built packages you then have to source it into you shell:\
`source ./install/setup.zsh` (if ros can't find things, try running this command)

There are two ways to start files in ROS.\
`ros2 run lhw_vision YoutubeLoader` will start a single file/node. Useful for small-scale testing.\
`ros2 launch lhw_vision vision.launch.py` will start many nodes (in this case vision).


### Run the whole system
The whole system can now be readied by running:\
`docker-compose up`

To only ready lhw_qi (for example):\
`docker-compose up lhw_qi`

This will run the `waiter` script in the container(s). It does two things:
- Keep the container running.
- Spy on the terminal output of any process started outside the container using the `cmd` script.

The `cmd` script is a tool I made to run processes inside the containers at the same time.\
The output log will be redirected to the waiter processes.\
Running commands can be canceled using *Ctrl-C* in *cmd*.
To run the commands in parallel instead of sequentially, add the `--p` flag after cmd.

The following command will build in each running container whose name regex-matches (lhw) sequentially:\
`sudo ./cmd lhw colcon build --symlink-install`


If you for example wanted to list all topics from lhw_intelligence: \
`sudo ./cmd intel ros2 topic list`

To launch the whole system from outside the container, run the following command:\
`sudo ./launch`

<br><br>
**Now go make awesome code :D**

## VSCode specific instructions
With vscode we can use devcontainers to get code suggestions etc which makes programming easier.
(TODO: Make this clearer)

- Get docker: https://docs.docker.com/get-docker/ 
- Get vscode: https://code.visualstudio.com/ 
- Get the vscode extension: Remote containers

You may now develop remotely in the container, with all the benefits of developing in a full IDE.

Some of our packages are added manually to the PYTHONPATH environment variable. Vscode cannot see these packages.
To work around this we may add a .env file in the root directory of your workspace (probably /worskspace/liu-home-wreckers). It could look like this:

`` PYTHONPATH="/opt/ros/foxy/lib/python3.8/site-packages:/detectron2/projects/DensePose:/workspace:/workspace/yolov5:/workspace/deep-person-reid/"
``

