"""
    This is the main AI file for the whole of lhw. If you want to perform a task,
    you also have to launch one of the state machines in this package.
    In other words, this is a hub which reacts to sensory input.
"""

from lhw_intelligence.extra_behaviours import *

import py_trees
import py_trees_ros

import rclpy
from rclpy.node import Node

from lhw_interfaces.msg._goal import Goal
from lhw_interfaces.msg._feedback import Feedback
from lhw_interfaces.msg._result import Result
from lhw_interfaces.msg import RobotState

from std_msgs.msg import String
from lhw_interfaces.msg import Entities, Response, Event
from geometry_msgs.msg import Twist, Point, PoseStamped
from nav_msgs.msg import OccupancyGrid
from .help_functions.DataBase import DataBase

class BatteryCheckTree(Node):
    def __init__(self):
        super().__init__('battery_check_tree')
        self.log = self.get_logger()

        # Publishers
        self.robot_state_pub = self.create_publisher(RobotState, "robot_state", 10)

        # Subscribers
        self.battery_sub = self.create_subscription(DiagnosticArray, "diagnostics", self.battery_cb, 10)


        # BLACKBOARD_KEYS
        self.blackboard = py_trees.blackboard.Client(name="Battery")
        self.blackboard.register_key(key="percentage", access=py_trees.common.Access.WRITE)
        self.blackboard.register_key(key="low_percentage", access=py_trees.common.Access.WRITE)

        self.log.info("py_tree started")
        self.iterations = 0
        root = py_trees.composites.Sequence("Sequence")
        
        # THE ORDER IN THE LIST MATTERS!
        #root.add_children([high, med, low])
        
        #root2 = py_trees.composites.Parallel("Parallel")
        #root2.add_children([high, med, low])
        #diagnostics = py_trees_ros.subscribers.ToBlackboard(topic_name="/diagnostics", topic_type=DiagnosticArray, qos_profile=py_trees_ros.utilities.qos_profile_latched(), blackboard_variables={"diagnostics": "status"})
        print_diagnostics = PepperRest("Pepper Rest", self)

        root2 = py_trees.composites.Sequence("Sequence")
        root2.add_child(print_diagnostics)
        #root2 = create_root2()
        behaviour_tree = py_trees.trees.BehaviourTree(
            root=root2
        )
        
        #test = behaviours.create_root()

        print(py_trees.display.unicode_tree(root=root))
        behaviour_tree.setup(timeout=15)

        def pre_tick_spin(tree):
            rclpy.spin_once(self)

        def tick_print(tree):
            print(py_trees.display.unicode_tree(root=tree.root, show_status=True))
            print(self.blackboard)
        
        try:
            behaviour_tree.tick_tock(
                period_ms=500,
                number_of_iterations=py_trees.trees.CONTINUOUS_TICK_TOCK,
                pre_tick_handler=None,
                post_tick_handler=tick_print
            )
        except KeyboardInterrupt:
            behaviour_tree.interrupt()

    def battery_cb(self, data):
        self.log.info("CALLBACK")
        
        # How to get the data for the battery from the DiagnosticArray
        # data.status[len(data.status)-3].values
        #self.log.info(data)
        battery_index = len(data.status)-3
        percentage = float(data.status[battery_index].values[0].value)
        self.blackboard.percentage =percentage
        if percentage < 30:
            self.blackboard.low_percentage = True
        elif percentage > 30 + 5: # To prevent ping-pong behaviour
            self.blackboard.low_percentage = False


def main(args=None):
    rclpy.init(args=args)
    ai_node = BatteryCheckTree()

    rclpy.spin(ai_node)

    ai_node.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()
