from .behaviour import Behaviour
from ..help_functions.Map import Map
from std_msgs.msg import String
from lhw_interfaces.msg._result import Result
from nav_msgs.msg import OccupancyGrid
from geometry_msgs.msg import PoseStamped

import numpy as np
from scipy.spatial.distance import euclidean 

from PIL import Image 

import os



METER_PER_PIXEL = 0.01

class ExploreRoom(Behaviour):
    def __init__(self, node, args):
        super().__init__(node, args)
        self.node = node

        node.log.info("ExploreRoom started")

        # ------------- State's properties --------------
        self.state_done = False

        self.is_looking_for_specific_id = args.specific_id
        self.specific_id = args.id
        self.is_looking_for_specific_type = args.specific_type
        self.specific_type = args.type

        self.state_name = String()
        self.state_name.data = "EXPLORE_ROOM"

        self.goal_points = []
        self.start_point = None
        self.goal_reached = False
        self.current_goal = None
        self.map = None


        # Get goals and start position from drawing:
        if args.map_name:
            image = Image.open('src/lhw_intelligence/lhw_intelligence/maps/' + args.map_name) 
            
            self.map = Map(image, METER_PER_PIXEL)
            node.log.info("Searching for start and goals in map...")  
            self.start_point, self.goal_points = self.map.get_start_and_goal_points()      

        self.current_pos = self.start_point

        self.run_timer = self.node.create_timer(1, self.run_behaviour_loop) 

        #TEMPORARY
        self.moving_to_goal_counter = 3

    def run_behaviour_loop(self):

        all_goals_visited = self.goal_reached and not self.goal_points

        if self.state_done or self._behaviour_timeout() or all_goals_visited: 
            self.run_timer.cancel()
            self.stop_robot()
            self.send_result()
            return

        if self.goal_reached or self.current_goal is None:
            #Rotate pepper and look around

            self.current_goal = self.__get_next_goal(self.goal_points, self.current_pos)
            self.goal_points.remove(self.current_goal)
            self.node.log.info("New goal. Move to " + str(self.current_goal))  
            self.goal_reached = False
            
            goal_pose = self.map.pos_to_pose(self.current_goal)
            self.move_robot_to_pose( goal_pose )
        
        #TEMPORARY
        self.moving_to_goal_counter -= 1
        if not self.moving_to_goal_counter:
            self.goal_reached = True
            self.moving_to_goal_counter = 5
            self.current_pos = self.current_goal


        #self.get_next_point
        

    def entities_callback(self, data):
        
        for entity in data.entities:

            if self.is_looking_for_specific_id and entity.id != self.specific_id:
                self.state_done = True

            if self.is_looking_for_specific_type and entity.type != self.specific_type:
                self.state_done = True

            entity_info = {}
            entity_info["id"] = entity.id
            entity_info["type"] = entity.type
            entity_info["position"] = self._relative_xyz_to_global_position(entity.xyz)

            self._add_to_db("seen_entities", entity_info)


        

    def __get_next_goal(self, goal_points, current_pos):
        

        closest_goal = min(goal_points, key=lambda goal : euclidean(goal, current_pos))

        
        return closest_goal


    

