
import numpy as np
from geometry_msgs.msg import Pose

WHITE = np.array([255, 255, 255, 255])
BLACK = np.array([0, 0, 0, 255])
RED = np.array([255, 0, 0, 255])
BLUE = np.array([0, 0, 255, 255])

class Map:
    
    def __init__(self, map_image, resolution):

        self.grid = np.transpose(np.asarray(map_image).copy(), (1,0,2))
        self.resolution = resolution

    def get_start_and_goal_points(self):

        map_grid = self.grid.copy()
        goal_points = []
        start_point = None

        for x_pos, x_col in enumerate(map_grid):
            for y_pos, cell in enumerate(x_col):

                if np.array_equal(cell, RED): 

                    goal_x_pos, goal_y_pos = self.__get_center_of_box_in_map(map_grid, x_pos, y_pos, RED)
                    
                    goal_points.append((goal_x_pos, goal_y_pos))

                    goal_box_height = 2 * (goal_x_pos-x_pos)
                    goal_box_width = 2 * (goal_y_pos-y_pos)

                    map_grid = self.__remove_box_from_map(map_grid, x_pos, y_pos, goal_box_height, goal_box_width)

                if np.array_equal(cell, BLUE): 

                    start_x_pos, start_y_pos = self.__get_center_of_box_in_map(map_grid, x_pos, y_pos, BLUE)
                    
                    start_point = (start_x_pos, start_y_pos)

                    start_box_height = 2 * (start_x_pos-x_pos)
                    start_box_width = 2 * (start_y_pos-y_pos)

                    map_grid = self.__remove_box_from_map(map_grid, x_pos, y_pos, start_box_height, start_box_width)

        return start_point, goal_points

    def pos_to_pose(self, pos):
        x = pos[0]*self.resolution
        y = pos[1]*self.resolution
        pose = Pose()
        pose.position.x = x
        pose.position.y = y
        pose.position.z = 0.0

        return pose

    def map2_grid_to_occupancy_grid(self, map_grid):

        map = OccupancyGrid()
        map.info.resolution = METER_PER_PIXEL
        map.info.width = map_grid.shape[0]
        map.info.height = map_grid.shape[1]
        
        map_data = []

        for x_pos, row  in enumerate(map_grid):
            for y_pos, cell in enumerate(row): 
                if np.array_equal(cell, BLACK):
                    map_data.append(1)
                if np.array_equal(cell, WHITE):
                    map_data.append(0)

        map.data = map_data    

        return map 


    def __get_center_of_box_in_map(self, map_grid, x_pos, y_pos, color):

        diagonal_negihbour_position = (x_pos + 1, y_pos + 1)
        diagonal_negihbour_value = map_grid[diagonal_negihbour_position]

        while np.array_equal(diagonal_negihbour_value, color):
            diagonal_negihbour_position = (x_pos + 1, y_pos + 1)
            diagonal_negihbour_value = map_grid[diagonal_negihbour_position]
            x_pos += 1
            y_pos += 1
        
        end_x_pos = diagonal_negihbour_position[0]
        end_y_pos = diagonal_negihbour_position[1]

        center_of_goal_row = (end_x_pos + x_pos) / 2
        center_of_goal_col = (end_y_pos + y_pos) / 2

        return center_of_goal_row, center_of_goal_col 

    def __remove_box_from_map(self, map_grid, x_pos, y_pos, goal_box_height, goal_box_width):
        end_x_pos = int(x_pos + goal_box_height)
        end_y_pos = int(y_pos + goal_box_width)

        map_grid[x_pos:end_x_pos, y_pos:end_y_pos] = WHITE

        return map_grid