
import json
from .safety_check.py_tree_behaviours import IsDoorOpen, IsHeadTouched, MoveForward, MoveToPoint, RotateRobot, Standby, StopRobot
from .safety_check.pepper_action_behaviours import *
from .safety_check.scenario_trees import *
from .behaviour_copy import Behaviour
import py_trees

import py_trees_ros

import rclpy
from rclpy.node import Node
from rclpy.action import ActionClient

from lhw_interfaces.msg._goal import Goal
from lhw_interfaces.msg._feedback import Feedback
from lhw_interfaces.msg._result import Result
from diagnostic_msgs.msg._diagnostic_array import DiagnosticArray
from diagnostic_msgs.msg._diagnostic_status import DiagnosticStatus

from std_msgs.msg import String
from sensor_msgs.msg import Range
from naoqi_bridge_msgs.msg import HeadTouch
from lhw_interfaces.msg import Entities, Response, Event, RobotAction, FaceDetected, Parameter
from geometry_msgs.msg import Twist, Point, PoseStamped
from nav_msgs.msg import OccupancyGrid, Odometry
from nav2_msgs.action import NavigateToPose

def dict_insert_list_or_append(adict: dict, key, val) -> None:
    '''Insert a value in dict at key if one does not exist
    Otherwise, convert value to list and append
    '''
    if key in adict:
        adict[key].append(val)
        adict[key] = sorted(adict[key], key=lambda x: x[0], reverse= True)
    else:
        adict[key] = [val]

class Py_Tree(Node):
    def __init__(self):
        super().__init__('py_tree')
        self.log = self.get_logger()
        """
        TODO: Implement what is necessary of this
        # ------- Communication with SM ---------
        self.goal_subscriber = self.create_subscription(Goal, "sm_goal", self.new_goal, 10)
        self.feedback_publisher = self.create_publisher(Feedback, 'sm_feedback', 1)
        self.result_publisher = self.create_publisher(Result, 'sm_result', 1)

        # ------- Subscribers ---------
        gpt_sub = self.create_subscription(Response, 'gpt_response', self.gpt_response_callback, 1)

        # ------- Publishers ---------
        self.realative_goal_pose_publisher = self.create_publisher(PoseStamped, 'realative_goal_pose', 1)
        self.map = self.create_publisher(OccupancyGrid, 'map', 1)
        self.event_publisher = self.create_publisher(Event, 'nlp_event', 100)
        """ 

        # ------- Publishers ---------
        self.goal_pose_publisher = self.create_publisher(PoseStamped, 'goal_pose', 1)
        self.cmd_vel_publisher = self.create_publisher(Twist, 'cmd_vel', 1)
        self.move_to_publisher = self.create_publisher(Twist, "move_to", 10)
        self.say_publisher = self.create_publisher(String, 'say', 100)
        self.robot_action_pub = self.create_publisher(RobotAction, 'robot_action', 1)
        # ------- Subscribers ---------
        self.dialogflow_sub = self.create_subscription(Response, 'dialogflow_response', self.dialogflow_response_callback, 1)
        self.face_detected_sub = self.create_subscription(FaceDetected, 'face_detected', self.face_detected_callback, 10)
        self.head_touched_subscriber = self.create_subscription(HeadTouch, 'head_touch', self.head_touched_cb, 1)
        self.sonar_front_subscriber = self.create_subscription(Range, 'sonar/front', self.sonar_front_cb, 1)
        self.odom_subscriber = self.create_subscription(Odometry, "odom", self.odom_cb, 1)
        self.navigate_subscriber = self.create_subscription(Feedback, "navigate_to_pose", self.navigate_cb, 1)
        self.battery_sub = self.create_subscription(DiagnosticArray, "diagnostics", self.battery_cb, 10)

        #client server (/navigate_to_pose)
        self._action_client = ActionClient(self, NavigateToPose, 'navigate_to_pose')
        
        self.pixel_to_xyz_serv = self.create_client(PixelToXYZ, "pixel_to_xyz")


        # Yolo subscriber
        #self.entities_sub = self.create_subscription(Entities, 'entities', self.entities_callback, 1)
        

        self.blackboard = py_trees.blackboard.Client(name="bb")
        
        self.blackboard.register_key(key="battery_percentage", access=py_trees.common.Access.WRITE)
        self.blackboard.register_key(key="battery_low_warning", access=py_trees.common.Access.WRITE)
        
        self.blackboard.register_key(key="head_touched", access=py_trees.common.Access.WRITE)
        self.blackboard.register_key(key="door_open", access=py_trees.common.Access.WRITE)
        self.blackboard.register_key(key="in_motion", access=py_trees.common.Access.WRITE)
        self.blackboard.register_key(key="at_destination", access=py_trees.common.Access.WRITE)
        
        self.blackboard.register_key(key="entities", access=py_trees.common.Access.WRITE)
        self.blackboard.register_key(key="point_to_pixel", access=py_trees.common.Access.WRITE)

        self.blackboard.register_key(key="recognized_text", access=py_trees.common.Access.WRITE)
        self.blackboard.register_key(key="last_recognized_item", access=py_trees.common.Access.WRITE)
        self.blackboard.register_key(key="recognized_people", access=py_trees.common.Access.WRITE)
        self.blackboard.register_key(key="found_new_person", access=py_trees.common.Access.WRITE)
        self.blackboard.register_key(key="current_person", access=py_trees.common.Access.WRITE)
        
        self.blackboard.register_key(key="already_introduced_people", access=py_trees.common.Access.WRITE)
        # self.blackboard.register_key(key="goal_received", access=py_trees.common.Access.WRITE)
        
        self.blackboard.head_touched = False
        self.blackboard.door_open = False
        self.blackboard.in_motion = False
        self.blackboard.at_destination = False
        self.blackboard.entities = {}
        self.blackboard.point_to_pixel = []
        self.blackboard.recognized_text = ""
        self.blackboard.recognized_people = {}
        self.blackboard.last_recognized_item = None
        self.blackboard.found_new_person = False
        # self.blackboard.goal_received = False

        behaviour = Behaviour(self)
        #config = json.load(open("src/lhw_intelligence/lhw_intelligence/config.json"))

        self.log.info("py_tree started")
        sequence = py_trees.composites.Parallel(name="Sequence")

        #Test for rotating and stopping the robot. This works"
        """timer = py_trees.timers.Timer(name="Timer", duration=2)
        rotate_robot = RotateRobot(name="Rotate Robot", node=self)
        timer2 = py_trees.timers.Timer(name="Timer2", duration=10)
        stop_robot = StopRobot(name="Stop Robot", node=self)
        timer3 = py_trees.timers.Timer(name="Timer3", duration=10)"""
        #timer, rotate_robot, timer2, stop_robot, timer3


        #s_c_tree = safety_check_tree(self, config)
        # door_open = IsDoorOpen(name="Door open", node=self)
        # move_forward = MoveForward(name="Move Forward", node=self, x=1.0)
        wait = py_trees.timers.Timer(name="Wait", duration=15)
        # head_touched = IsHeadTouched(name="Head Touched", node=self)
        greet = Standby(name="Standby", node=self)

        #test = py_trees_ros.subscribers.ToBlackboard(topic_name="diagnostics", topic_type=DiagnosticArray, blackboard_variables={"diagnostics": None}, qos_profile=10)
        
        """ To help test the navstack """
        move_to_safety_point = MoveToPoint(name="Move To Safety Point", node=self, pose=behaviour.position_to_pose({"x": 1.0,"y": 0.0,"z": 0.0}))
        standby2 = Standby(name="Standby", node=self)
        nav_test_seq = py_trees.composites.Sequence(name="Nav Test Seq")
        nav_test_seq.add_children([move_to_safety_point, standby2])


        action = RobotAction()
        action.type = RobotAction.POINT_ABS_STRAIGHT
        action.right_arm = False
        
        pepper_point_straight_l = PepperAction(name="Point", node=self, action=action)
        
        action2 = RobotAction()
        action2.type = RobotAction.POINT_ABS_STRAIGHT
        action2.right_arm = True
        pepper_point_straight_r = PepperAction(name="Pepper Point", node= self, action=action2)
        battery = IsBatteryLow(name="Battery Low")
    

        par = py_trees.composites.Parallel(name="TEST")

        par.add_children([pepper_point_straight_r, pepper_point_straight_l])

        rest = PepperRest(name="REST", node=self)
        

        timer = py_trees.timers.Timer(name="Timer", duration=1)
        rotate_robot = RotateRobot(name="Rotate Robot", node=self)
        timer2 = py_trees.timers.Timer(name="Timer2", duration=1)
        stop_robot = StopRobot(name="Stop Robot", node=self)
        safety_off = PepperSetSafety(name="Safety Off", node=self, safety_on = False)

        rotate_seq = py_trees.composites.Sequence(name="ROTATE")
        rotate_seq.add_children([safety_off, rotate_robot])

        say_seq = py_trees.composites.Sequence(name="SAY")
        say = Say(name="Say", node=self, message="Hi!")
        say_seq.add_children([say, greet])

        center_head = PepperCenterHead(name="Center Head", node=self)

        wake_up = PepperWakeUp(name="Wake Up", node=self)
        headbob = py_trees.composites.Sequence(name="Head Bob")
        tilt_up = PepperTiltHead(name="Tilt Up", node=self, tilt_up=True)
        tilt_down = PepperTiltHead(name="Tilt Down", node=self, tilt_up=False)
        headbob.add_children([tilt_down, timer, tilt_up, timer2])

        idle_off = IdleOff(name="Idle Off", node=self)
        #breath = PepperBreath(name="Breath", node=self)

        #idle_on = IdleOn(name="Idle On", node=self)


        roll_elbow = PalmOpen(name="Roll Elbow", node=self, right_hand = True)
        close_elbow = PalmClose(name="Close Elbow", node=self, right_hand = True)

        confirm = ConfirmSay(name="Confirm", node=self)

        point = py_trees.composites.Sequence(name="POINT")
        find_person = WaitForPerson(name="Find person", node=self)
        getPixel = GetFacePixelCoordinate(name="getPixel", node=self)
        pointAt = PointAtPixel(name="Point At", node=self)
        point.add_children([find_person, getPixel, pointAt])

        self.behaviour_tree = py_trees.trees.BehaviourTree(root=point)
      
        self.behaviour_tree.setup(timeout=15)

        
        """ Run before every tick to clear all backlogged callbacks
        """
        def pre_tick_spin(tree):
            rclpy.spin_once(self)

        def tick_print(tree):
            #print(py_trees.display.unicode_tree(root=tree.root, show_status=True))
            #print(self.blackboard)
            pass
        
        try:
            self.behaviour_tree.tick_tock(
                period_ms=500,
                number_of_iterations=py_trees.trees.CONTINUOUS_TICK_TOCK,
                pre_tick_handler=pre_tick_spin,
                post_tick_handler=tick_print
            )
        except KeyboardInterrupt:
            self.behaviour_tree.interrupt()
        
    def head_touched_cb(self, head_touched_msg):
        print("head touched!")
        self.blackboard.head_touched = True

    def sonar_front_cb(self, sonar_msg):
        #print("Sonar Front Callback", flush=True)
        if sonar_msg.range > 1.0:
            self.blackboard.door_open = True

    def odom_cb(self, odom_msg):
        #print("Odom Callback", flush=True)
        linear = odom_msg.twist.twist.linear
        if (linear.x != 0 or linear.y != 0 or linear.z != 0):
            self.blackboard.in_motion = True
        else:
            self.blackboard.in_motion = False
    
    def navigate_cb(self, nav_msg):
        # Find out what data looks like of nav_msg and how we can write the different things

        # Tell the blackboard if we have reached our destination
        # TODO: Can this be exact or should we have a buffer? like 0.1 or something
        if nav_msg.distance_remaining == 0.0:
            self.blackboard.at_destination = True
        else:
            self.blackboard.at_destination = False

        # TODO: What to do if it fails?
        # Tell the blackboard if we're currently in motion
        if nav_msg.in_motion:
            self.blackboard.in_motion = True
        else:
            self.blackboard.in_motion = False

        # # FUNKAR EJ!
        # if nav_msg.goal_received:
        #     self.blackboard.goal_received = True
        # else:
        #     self.blackboard.goal_received = False
    
    def battery_cb(self, data):
        # How to get the data for the battery from the DiagnosticArray
        # data.status[len(data.status)-3].values
        PERCENTAGE_LIM = 30
        battery_index = len(data.status)-3
        percentage = float(data.status[battery_index].values[0].value)
        self.blackboard.battery_percentage = percentage
        if percentage < PERCENTAGE_LIM:
            self.blackboard.battery_low_warning = True
        elif percentage > PERCENTAGE_LIM + 5: # To prevent ping-pong behaviour
            self.blackboard.battery_low_warning = False

    # Checking for gaps in the sofa to point at
    # TODO: Test this
    def entities_cb(self, yolo_data):
        width = yolo_data.source_width
        dict_insert_list_or_append(self.blackboard.entities, "picture_width", width)
        yolo_dict = yolo_data.entities
        for id in yolo_dict:
            if yolo_dict[id].type == "couch" or yolo_dict[id].type == "person":
                #TODO: Maybe need to check size of bbox so that we dont insert people from the background
                #TODO: Test if this becomes nestled list for several persons
                dict_insert_list_or_append(self.blackboard.entities, yolo_dict[id].type, [yolo_dict[id].bbox[0], yolo_dict[id].bbox[2]])

    def dialogflow_response_callback(self, data):

        self.blackboard.recognized_text = data.recognized_text
        if data.parameters:
            """for parameter in data.parameters:
                params[parameter.key] = parameter.value"""
            param = data.parameters[0]
            
            self.blackboard.last_recognized_item = param # TODO: Test this
    
    def face_detected_callback(self, face_msg):
        print("FaceDetected callback!")
        if not face_msg:
            return
        print(face_msg)
        # Add the person and initialize different items if not recognized since earlier
        if face_msg.id not in self.blackboard.recognized_people:
            self.blackboard.found_new_person = True
            self.blackboard.recognized_people[face_msg.id] = {
                "z_index" : face_msg.z_index, 
                "name" : None, 
                "drinks" : None, 
                "age" : face_msg.age, 
                "face_center" : [face_msg.face_center_xy[0], face_msg.face_center_xy[1]]
            }
        # Update the z_index of the detected person
        else:
            self.blackboard.recognized_people[face_msg.id]["z_index"] = face_msg.z_index
            # Only change age if it is lower than previous, prevents age bouncing around
            if face_msg.age < self.blackboard.recognized_people[face_msg.id]["age"]:
                self.blackboard.recognized_people[face_msg.id]["age"] = face_msg.age
        print(self.blackboard.recognized_people)

def main(args=None):
    rclpy.init(args=args)
    ai_node = Py_Tree()

    rclpy.spin(ai_node)

    ai_node.destroy_node()
    rclpy.shutdown()

if __name__ == '__main__':
    main()
