import py_trees
from .actions import Actions

#from naoqi_bridge_msgs.msg import HeadTouch

from ..behaviour_copy import Behaviour #TODO: Move behaviours to here

#TODO: Tidy this up when finished
from std_msgs.msg import Bool, String, Empty, Int16, Header, Int32
from builtin_interfaces.msg import Time
import time, random
from lhw_interfaces.msg import Entities, Response, Entity, Result
from nav2_msgs.action import NavigateToPose
from geometry_msgs.msg import Twist, Pose, PoseStamped
from diagnostic_msgs.msg._diagnostic_array import DiagnosticArray
from diagnostic_msgs.msg._diagnostic_status import DiagnosticStatus
import numpy as np

""" Rotates the robot counter (?) clockwise.
"""
class RotateRobot(py_trees.behaviour.Behaviour):
    def __init__(self, name, node) -> None:
        super(RotateRobot, self).__init__(name)
        self.node = node

    def setup(self, **kwargs):
        self.actions = Actions(self.node)

    def initialise(self):
        pass
        
    def update(self):
        self.actions.rotate_robot()
        return py_trees.common.Status.RUNNING
        """if (Robotidle):
            self.node.log.info("Rotation Succeeded")
            return py_trees.common.Status.SUCCESS
        else:
            return py_trees.common.Status.RUNNING"""
        
    def terminate(self, new_status):
        self.node.log.info("Emergancy Stop")
        self.actions.stop_robot()


""" Stops the robot
"""
class StopRobot(py_trees.behaviour.Behaviour):
    def __init__(self, name, node) -> None:
        super(StopRobot, self).__init__(name)
        self.node = node

    def setup(self, **kwargs):
        self.actions = Actions(self.node)

    def initialise(self):
        self.actions.stop_robot()

    def update(self):
        return py_trees.common.Status.SUCCESS
        
    def terminate(self, new_status):
        pass

""" mocking navigation for testing
"""
class MockNavigation(py_trees.behaviour.Behaviour):
    def __init__(self, name, node) -> None:
        super(MockNavigation, self).__init__(name)
        self.node = node

    def setup(self, **kwargs):
        pass

    def initialise(self):
        pass

    def update(self):
        return py_trees.common.Status.SUCCESS
        rand = random.random()
        if rand < 0.3:
            return py_trees.common.Status.FAILURE
        elif rand < 0.6:
            return py_trees.common.Status.RUNNING
        else:
            return py_trees.common.Status.SUCCESS
        
    def terminate(self, new_status):
        pass

""" Moves the robot forward x meters
"""
class MoveForward(py_trees.behaviour.Behaviour):
    def __init__(self, name, node, x: float) -> None:
        super(MoveForward, self).__init__(name)
        self.node = node
        self.x = x

    def setup(self, **kwargs):
        self.action = Actions(self.node)
        self.blackboard = py_trees.blackboard.Client(name="bb")
        self.blackboard.register_key(key="in_motion", access=py_trees.common.Access.READ)
        

    def initialise(self):

        self.node.log.info(f"Moving forward {self.x}m")
        msg = Twist()
        speed = 1.0
        msg.linear.x = speed
        msg.linear.y = 0.0
        msg.linear.z = 0.0
        msg.angular.z = 0.0
        self.node.cmd_vel_publisher.publish(msg)
        self.start_time = time.time()
        self.time_diff = self.x * 2 * speed + 1
        print(self.start_time)


    def update(self):
        
        if float(time.time() - self.start_time) < self.time_diff:
            return py_trees.common.Status.RUNNING
        else:
            return py_trees.common.Status.SUCCESS

        """
        if (self.blackboard.in_motion):
            return py_trees.common.Status.RUNNING
        else:
            return py_trees.common.Status.SUCCESS
        """


        #TODO: Få tag i data från navigation, när är vi stilla, har vi nått slutmålet

        """if (<idle> && <in location>):
            return py_trees.common.Status.SUCCESS
        elif (<idle> && <not at localtion>):        
            return py_trees.common.Status.FAILURE
        else:
            return py_trees.common.Status.RUNNING"""
        
    def terminate(self, new_status):
        self.node.log.info(f"Emergency Stop")
        self.action.stop_robot()
        


""" Checks using ultra sound if the door in front of the robot is open or not
"""
class IsDoorOpen(py_trees.behaviour.Behaviour):
    def __init__(self, name, node) -> None:
        super(IsDoorOpen, self).__init__(name)
        self.node = node

    def setup(self, **kwargs):
        self.blackboard = py_trees.blackboard.Client(name="bb")
        self.blackboard.register_key(key="door_open", access=py_trees.common.Access.READ)

    def update(self):
        if self.blackboard.door_open == True:
            print("Door is open!")
            return py_trees.common.Status.SUCCESS
        else:
            return py_trees.common.Status.RUNNING

    def terminate(self, new_status):
        pass


""" Move the robot to a specified location
"""
class MoveToPoint(py_trees.behaviour.Behaviour):
    def __init__(self, name, node, pose: Pose) -> None:
        super(MoveToPoint, self).__init__(name)
        self.node = node
        self.pose = pose
        self.result = False
        self.action_client = self.node._action_client

    def setup(self, **kwargs):
        self.action = Actions(self.node)
        # TODO: How these are setup depends on what data we get from NavigateToPose
        self.blackboard = py_trees.blackboard.Client(name="bb")
        self.blackboard.register_key(key="in_motion", access=py_trees.common.Access.READ)
        self.blackboard.register_key(key="at_destination", access=py_trees.common.Access.READ)
        self.blackboard.register_key(key="goal_received", access=py_trees.common.Access.READ)
        # TODO: Do something for checking for failures

    def initialise(self):
        self.action_client.wait_for_server()

        goal = NavigateToPose.Goal()

        poseStamped = PoseStamped()
        poseStamped.pose = self.pose
        goal.pose = poseStamped
        
        header = Header()
        stamp = Time()
        stamp.sec = int(time.time() % 60)
        stamp.nanosec = 0
        header.stamp = stamp
        header.frame_id = "id"

        #goal.behaviour_tree = "write behaviour tree name"

        self.future = self.action_client.send_goal_async(goal)
        self.future.add_done_callback(self.result_callback)
        

    def update(self):
        if self.result:
            return py_trees.common.Status.SUCCESS
        else:
            return py_trees.common.Status.RUNNING
        
    def terminate(self, new_status):
        self.node.log.info(f"Emergency Stop")
        self.action.stop_robot()
    
    def result_callback(self, future):
        self.result = True
        #TODO: check if it failed or not


""" Checks if the head has been tapped
    This is a blocking behaviour
"""
class IsHeadTouched(py_trees.behaviour.Behaviour):
    def __init__(self, name, node) -> None:
        super(IsHeadTouched, self).__init__(name)
        self.node = node

    def setup(self, **kwargs):
        self.blackboard = py_trees.blackboard.Client(name="bb")
        self.blackboard.register_key(key="head_touched", access=py_trees.common.Access.READ)
        self.blackboard.register_key(key="head_touched", access=py_trees.common.Access.WRITE)

    def update(self):
        print("Head Tick")
        if self.blackboard.head_touched == True:
            self.node.log.info(f"Head has been touched")
            self.blackboard.head_touched = False #TODO: Test other ways to do this
            return py_trees.common.Status.SUCCESS
        else:
            return py_trees.common.Status.RUNNING

    def terminate(self, new_status):
        pass

class ResetHeadTouched(py_trees.behaviour.Behaviour):
    def __init__(self, name, node) -> None:
        super(ResetHeadTouched, self).__init__(name)
        self.node = node

    def setup(self, **kwargs):
        self.blackboard = py_trees.blackboard.Client(name="bb")
        self.blackboard.register_key(key="head_touched", access=py_trees.common.Access.WRITE)

    def initialize(self):
        pass
    def update(self):
        self.blackboard.head_touched = False
        return py_trees.common.Status.SUCCESS

    def terminate(self, new_status):
        pass

""" Standby 
"""
class Standby(py_trees.behaviour.Behaviour):
    def __init__(self, name, node) -> None:
        super(Standby, self).__init__(name)
        self.node = node

    def setup(self, **kwargs):
        pass

    def initialise(self):
        # Initiate standby pose
        pass

    def update(self):
        return py_trees.common.Status.RUNNING

    def terminate(self, new_status):
        pass

""" 
"""
class CalculateGapPointLocation(py_trees.behaviour.Behaviour):
    def __init__(self, name, node) -> None:
        super(CalculateGapPointLocation, self).__init__(name)
        self.node = node

    def setup(self, **kwargs):
        self.blackboard = py_trees.blackboard.Client(name="bb")
        self.blackboard.register_key(key="entities", access=py_trees.common.Access.READ)
        self.blackboard.register_key(key="point_to_pixel", access=py_trees.common.Access.WRITE)

    def initialise(self):
        pass
    def update(self):
        
        # TODO: Test this
        GAP_LIM = 50
        couch = {"left":self.blackboard.entites["couch"][0], "right": self.blackboard.entites["couch"][1]}
        persons = self.blackboard.entites["person"]
        spaces = []

        # Calculate all gaps in the sofa
        for i in range(len(persons)):
            left = persons[i][0]
            right = persons[i][1]

            if i == 0:
                spaces.append({"left": couch["left"], "right": left, "gap": couch["left"] - left})
            else:
                prev_right = persons[i-1][1]
                spaces.append({"left": prev_right, "right": left, "gap": prev_right - right})
        right = persons[len(persons)-1][1]
        spaces.append({"left": right, "right": couch["right"], "gap": right - couch["right"]})

        largest_gap = max(spaces, key= lambda x: x["gap"])

        # Check if gap is actually big enough
        if largest_gap["gap"] >= GAP_LIM:
            # Gets middle location of the left and right x pixel
            mid_x = (largest_gap["right"] + largest_gap["left"])/2
            Y = 150 # Decide for value (if even needed)
            self.blackboard.point_to_pixel = [mid_x, Y]
            return py_trees.common.Status.SUCCESS
        else:
            return py_trees.common.Status.FALIURE

    def terminate(self, new_status):
        pass

"""
    Gets and saves the pixel coordinate of the first person found which has not 
    been introduced yet and adds them to the list of introduced people.
    If such a person cannot be found, halts the tree
"""
class GetFacePixelCoordinate(py_trees.behaviour.Behaviour):
    def __init__(self, name) -> None:
        super(GetFacePixelCoordinate, self).__init__(name)
    
    def setup(self, **kwargs):
        self.blackboard = py_trees.blackboard.Client(name="bb")
        self.blackboard.register_key(key="already_introduced_people", access=py_trees.common.Access.READ)
        self.blackboard.register_key(key="recognized_people", access=py_trees.common.Access.READ)
        self.blackboard.register_key(key="current_person", access=py_trees.common.Access.READ)
        self.blackboard.register_key(key="already_introduced_people", access=py_trees.common.Access.WRITE)
        self.blackboard.register_key(key="point_to_pixel", access=py_trees.common.Access.WRITE)

    def initialise(self):
        pass

    def update(self):
        for id in self.blackboard.recognized_people:
            #name = self.blackboard.recognized_people[id]["name"]
            #if name not in self.blackboard.already_introduced_people and name != self.blackboard.current_person:
            x = self.blackboard.recognized_people[id]["face_center"][0]
            y = self.blackboard.recognized_people[id]["face_center"][1]
                
            self.blackboard.point_to_pixel = [x, y]
                #self.blackboard.already_introduced_people.append(name)
            return py_trees.common.Status.SUCCESS
        return py_trees.common.Status.RUNNING
        
    def terminate(self, new_status):
        pass

"""
    Checks if everyone that is currently saved has been introduced, 
    if so return SUCCESS otherwise return FAILURE
"""
class IsEveryoneIntroduced(py_trees.behaviour.Behaviour):
    def __init__(self, name) -> None:
        super(IsEveryoneIntroduced, self).__init__(name)
    
    def setup(self, **kwargs):
        self.blackboard = py_trees.blackboard.Client(name="bb")
        self.blackboard.register_key(key="already_introduced_people", access=py_trees.common.Access.READ)
        self.blackboard.register_key(key="recognized_people", access=py_trees.common.Access.READ)
        self.blackboard.register_key(key="current_person", access=py_trees.common.Access.READ)
    def initialise(self):
        pass

    def update(self):
        # Succeed when everyone has been introduced
        for id in self.blackboard.recognized_people:
            id_name = self.blackboard.recognized_people[id]
            if id_name not in self.blackboard.already_introduced_people and id_name != self.blackboard.current_person:
                return py_trees.common.Status.FAILURE
        return py_trees.common.Status.SUCCESS

    def terminate(self, new_status):
        pass

class IsBatteryLow(py_trees.behaviour.Behaviour):
    def __init__(self, name) -> None:
        super(IsBatteryLow, self).__init__(name)
    
    def setup(self, **kwargs):
        self.blackboard = py_trees.blackboard.Client(name="Battery")
        self.blackboard.register_key(key="low_percentage", access=py_trees.common.Access.READ)

    def initialise(self):
        pass

    def update(self):
        if self.blackboard.low_percentage:
            return py_trees.common.Status.SUCCESS
        else:
            return py_trees.common.Status.FAILURE

    def terminate(self, new_status):
        pass

class Say(py_trees.behaviour.Behaviour):
    def __init__(self, name, node, message: str) -> None:
        super(Say, self).__init__(name)
        self.node = node
        self.message = message
        self.say_pub = self.node.say_publisher
    
    def setup(self, **kwargs):
        self.msg = String()
        self.msg.data = self.message
        pass 
    def initialise(self):
        self.say_pub.publish(self.msg)

    def update(self):
        return py_trees.common.Status.SUCCESS

    def terminate(self, new_status):
        pass

class WaitForItem(py_trees.behaviour.Behaviour):
    def __init__(self, name, node, key: str) -> None:
        super(WaitForItem, self).__init__(name)
        self.node = node
        self.key = key
    
    def setup(self, **kwargs):
        self.blackboard = py_trees.blackboard.Client(name="bb")
        self.blackboard.register_key(key="last_recognized_item", access=py_trees.common.Access.READ)
    def initialise(self):
        pass

    def update(self):
        print(self.blackboard.last_recognized_item)
        if self.blackboard.last_recognized_item and self.blackboard.last_recognized_item.key == self.key:
            return py_trees.common.Status.SUCCESS
        return py_trees.common.Status.RUNNING

    def terminate(self, new_status):
        pass

class RemoveItem(py_trees.behaviour.Behaviour):
    def __init__(self, name, node) -> None:
        super(RemoveItem, self).__init__(name)
        self.node = node
    
    def setup(self, **kwargs):
        self.blackboard = py_trees.blackboard.Client(name="bb")
        self.blackboard.register_key(key="last_recognized_item", access=py_trees.common.Access.WRITE)

    def initialise(self):
        self.blackboard.last_recognized_item = None

    def update(self):
        self.blackboard.last_recognized_item = None
        return py_trees.common.Status.SUCCESS

    def terminate(self, new_status):
        pass

class WaitForPerson(py_trees.behaviour.Behaviour):
    def __init__(self, name, node) -> None:
        super(WaitForPerson, self).__init__(name)
        self.node = node
    
    def setup(self, **kwargs):
        self.blackboard = py_trees.blackboard.Client(name="bb")
        self.blackboard.register_key(key="found_new_person", access=py_trees.common.Access.READ)
        self.blackboard.register_key(key="found_new_person", access=py_trees.common.Access.WRITE)

    def initialise(self):
        pass

    def update(self):
        if self.blackboard.found_new_person:
            self.blackboard.found_new_person = False
            return py_trees.common.Status.SUCCESS
        return py_trees.common.Status.RUNNING

    def terminate(self, new_status):
        pass

class ConfirmSay(py_trees.behaviour.Behaviour):
    def __init__(self, name, node) -> None:
        super(ConfirmSay, self).__init__(name)
        self.node = node
        self.say_pub = self.node.say_publisher # FOR TESTING
    
    def setup(self, **kwargs):
        self.blackboard = py_trees.blackboard.Client(name="bb")
        self.blackboard.register_key(key="recognized_text", access=py_trees.common.Access.READ)
        self.blackboard.register_key(key="recognized_text", access=py_trees.common.Access.WRITE)
        self.blackboard.register_key(key="last_recognized_item", access=py_trees.common.Access.READ)
        self.blackboard.register_key(key="last_recognized_item", access=py_trees.common.Access.WRITE)
    def initialise(self):
        pass

    def update(self):
        msg = String() # TESTING

        if "yes" in self.blackboard.recognized_text.lower():
            msg.data = "Confirmed"
            self.say_pub.publish(msg)
            self.blackboard.recognized_text = ""
            return py_trees.common.Status.SUCCESS
        elif "no" in self.blackboard.recognized_text.lower():
            msg.data = "No"
            self.say_pub.publish(msg)
            self.blackboard.recognized_text = ""
            self.blackboard.last_recognized_item = None # TODO: Is this too "hard-coded"?
            return py_trees.common.Status.FAILURE
        return py_trees.common.Status.RUNNING

    def terminate(self, new_status):
        pass

class AddItemToPerson(py_trees.behaviour.Behaviour):
    def __init__(self, name, node) -> None:
        super(AddItemToPerson, self).__init__(name)
        self.node = node
    
    def setup(self, **kwargs):
        self.blackboard = py_trees.blackboard.Client(name="bb")
        self.blackboard.register_key(key="recognized_people", access=py_trees.common.Access.WRITE)
        self.blackboard.register_key(key="last_recognized_item", access=py_trees.common.Access.READ)
        self.blackboard.register_key(key="last_recognized_item", access=py_trees.common.Access.WRITE)
        self.blackboard.register_key(key="last_recognized_item", access=py_trees.common.Access.WRITE)
        self.blackboard.register_key(key="current_person", access=py_trees.common.Access.WRITE)
    def initialise(self):
        # Get the closest person to Pepper
        closest_person = min(self.blackboard.recognized_people.values(), key=lambda x: x["z_index"])
        param = self.blackboard.last_recognized_item

        closest_person[param.key] = param.value

        # Save the name of the current person being introduced
        if param.key == "name":
            self.blackboard.current_person = param.value

        self.blackboard.last_recognized_item = None

        # TODO: Code if multiple parameters can be passed through last_recognized_item
        """for key in params:
            # In case we ever use other parameters with dialogflow
            if key in ["name", "drink", "age"]:
                value = params[key]
                closest_person[key] = value"""
    

    def update(self):
       return py_trees.common.Status.SUCCESS

    def terminate(self, new_status):
        pass