import py_trees
from .py_tree_behaviours import *
from .pepper_action_behaviours import *
from lhw_intelligence.behaviour_copy import Behaviour

""" Py_tree for the safety check procedure
"""
def safety_check_tree(node, config: dict):
    behaviour = Behaviour(node)
    safety_pose = behaviour.position_to_pose(config["safety_point"])
    exit_pose = behaviour.position_to_pose(config["exit"])

    root = py_trees.composites.Sequence(name="Standby Sequence")
    
    safety_check_sequence = py_trees.composites.Sequence(name="Safety Check Sequence")
    is_door_open = IsDoorOpen(name="Is Door Open", node=node)
    mock_forward = MockNavigation(name="Mock Forward Move", node=node)
    #move_forward = MoveForward(name="Move Forward", node=node, x=1.0)
    wait = py_trees.timers.Timer(name="Wait", duration=2)
    mock_safety = MockNavigation(name="Mock Safety Point Move", node=node)
    #move_to_safety_point = MoveToPoint(name="Move to Point",node=node, pose=safety_pose)
    head_touched_selector_wait = py_trees.composites.Selector(name="Head Touched Selector Wait")
    is_head_touched = IsHeadTouched(name="Is Head Touched")
    head_touched_retry = Standby(name="Head Touched Retry")
    #move_to_exit = MoveToPoint(name="Move to Exit", node=node, pose=exit_pose)
    mock_exit = MockNavigation(name="Mock Exit Move", node=node)
    standby = Standby(name="Standby", node=node)

    head_touched_selector_wait.add_children([is_head_touched, head_touched_retry])



    safety_check_sequence.add_children([is_door_open, mock_forward, wait, mock_safety, head_touched_selector_wait, mock_exit])
    root.add_children([safety_check_sequence, standby])

    return root

""" Py_tree for the safety check procedure
"""
def enter_arena(node):

    root = py_trees.composites.Sequence(name="Standby Sequence")
    
    enter_arena_sequence = py_trees.composites.Sequence(name="Enter Arena Sequence")
    #tell_referee_to_open_door = Say(name="Open The Door", node=node, message="Please open the door for me")
    is_door_open = IsDoorOpen(name="Is Door Open", node=node)
    #mock_forward = MockNavigation(name="Mock Forward Move", node=node)
    move_forward = MoveForward(name="Move Forward", node=node, x=1.0)
    timer5 = py_trees.timers.Timer(name="Wait", duration=5)
    #rotate = RotateRobot(name="Rotate", node=node) # TODO: Add radian

    standby = Standby(name="Standby", node=node)


    enter_arena_sequence.add_children([is_door_open, move_forward, timer5])
    root.add_children([enter_arena_sequence, standby])

    return root


""" 
    Behaviour tree to greet new people and get what their name and favourite drink is.
    
    Required running nodes:
        - face_analysis node & deepface server
        - Dialogflow
        - Naoqi_driver & animated speech & microphone
    Reads/Writes to blackboard variables:
        - last_recognized_item
        - recognized_people
        - recognized_text
"""
def greet_person(node, repeat_message="I'm sorry I didn't quite catch that,"):    
    greet_person_subtree = py_trees.composites.Sequence(name="Greet Person Subtree")
    find_person = WaitForPerson(name="Find person", node=node)
    greet = Say(name="Greet", node=node, message="Hello, welcome!")
    name_wait_sel = py_trees.composites.Selector(name="Name Wait Sel")
    get_name_seq = py_trees.composites.Sequence(name="Get Name Seq")
    ask_for_name = Say(name="Ask For Name", node=node, message="What's your name?")
    wait_for_name = WaitForItem(name="Wait For Name", node=node, key="name")
    confirm_name = ConfirmSay(name="Confirm Name", node=node)
    add_name = AddItemToPerson(name="Add Name", node=node)
    ask_name_again_seq = py_trees.composites.Sequence(name="Ask Name Again Seq")
    ask_name_again = Say(name="Ask Name Again", node=node, message=repeat_message)
    fail_name = py_trees.behaviours.Failure(name='Name Failure')
    name_retry = Standby(name="Name Retry", node=node)
    
    drink_wait_sel = py_trees.composites.Selector(name="Drink Wait Sel")
    get_drink_seq = py_trees.composites.Sequence(name="Get Drink Seq")
    ask_for_favorite_drink = Say(name="Ask For Favorite Drink", node=node, message="What's your favorite drink?")
    wait_for_drink = WaitForItem(name="Wait For Drink", node=node, key="drinks")
    confirm_drink = ConfirmSay(name="Confirm Drink", node=node)
    add_drink = AddItemToPerson(name="Add Drink", node=node)
    ask_drink_again_seq = py_trees.composites.Sequence(name="Ask Name Again Seq")
    ask_drink_again = Say(name="Ask Drink Again", node=node,message=repeat_message)
    fail_drink = py_trees.behaviours.Failure(name='Drink Failure')
    
    drink_retry = Standby(name="Drink Retry", node=node)

    completed_greet = Say(name="Complete", node=node, message="Thank you! That should be all I need to know")
    greet_complete_delay = py_trees.timers.Timer(name="Greet Complete Delay", duration=1)
    fin = Say(name="Fin", node=node, message="FINISHED!")
    
    
    
    drink_wait_sel.add_children([get_drink_seq, ask_drink_again_seq, drink_retry])
    
    get_drink_seq.add_children([ask_for_favorite_drink, wait_for_drink, confirm_drink, add_drink])
    
    get_name_seq.add_children([ask_for_name, wait_for_name, confirm_name, add_name])
    
    greet_person_subtree.add_children([find_person, greet, name_wait_sel, drink_wait_sel, completed_greet, greet_complete_delay, fin])
    
    ask_name_again_seq.add_children([ask_name_again, fail_name])

    ask_drink_again_seq.add_children([ask_drink_again, fail_drink])

    name_wait_sel.add_children([get_name_seq, ask_name_again_seq, name_retry])
    
    return greet_person_subtree

def receptionist(node):    
    root = py_trees.composites.Sequence(name="Root")
    is_head_touched = IsHeadTouched(name="Is Head Touched", node=node)
    wait = py_trees.timers.Timer(name="Wait", duration=5)
    tell_referee_to_open_door = Say(name="Tell Referee To Open Door", node=node)
    greet_person_subtree = greet_person(node=node)
    tell_person_to_follow = Say(name="Tell Person To Follow", node=node)
    look_down = PepperTiltHead(name="Look Down", node=node)
    move_to_sofa = MoveToPoint(name="Move To Sofa", node=node)
    pepper_center_head = PepperCenterHead(name="Pepper Center Head", node=node)
    at_sofa_seq = py_trees.composites.Sequence(name="At Sofa Seq")
    intros_wait_sel = py_trees.composites.Selector(name="Intros Wait Sel")
    all_people_introduced = IsEveryoneIntroduced(name="All People Introduced", node=node)
    introduce_person = py_trees.composites.Sequence(name=" Introduce person")
    get_face_pixel = GetFacePixelCoordinate(name="Get Face Pixel", node=node)
    point_at_person = PointAtPixel(name="Point At Person", node=node)
    say_name_of_person = Say(name="Say Name Of Person", node=node)
    const_fail = py_trees.behaviours.Failure(name="Const Fail")
    
    introduce_retry = Standby(name="Introduce Retry", node=node)
    
    empty_seat_sel = py_trees.composites.Selector(name="Empty Seat Sel")
    successful_seating = py_trees.composites.Sequence(name="Successful Seating")
    get_couch_gap_pixel = CalculateGapPointLocation(name="Get Couch Gap Pixel", node=node)
    point_at_empty_seat = PointAtPixel(name="Point At Empty Seat", node=node)
    tell_person_to_sit = Say(name="Tell Person To Sit", node=node)
    
    unsuccessful_seating = py_trees.composites.Sequence(name="Unsuccessful Seating")
    say_there_is_no_seat = Say(name="Say There Is No Seat", node=node)
    
    move_to_door = MoveToPoint(name="Move To Door", node=node)
    
    at_sofa_seq.add_children([intros_wait_sel, empty_seat_sel])
    
    empty_seat_sel.add_children([successful_seating, unsuccessful_seating])
    
    introduce_person.add_children([get_face_pixel, point_at_person, say_name_of_person, const_fail])
    
    intros_wait_sel.add_children([all_people_introduced, introduce_person, introduce_retry])
    
    root.add_children([is_head_touched, wait, tell_referee_to_open_door, greet_person_subtree, tell_person_to_follow, look_down, move_to_sofa, pepper_center_head, at_sofa_seq, move_to_door])
    
    successful_seating.add_children([get_couch_gap_pixel, point_at_empty_seat, tell_person_to_sit])
    
    unsuccessful_seating.add_child(say_there_is_no_seat)
    return root

"""
    For testing repeating behaviour with resetting of blackboard variables
"""
def test(node):    
    greet_person_subtree = py_trees.composites.Sequence(name="Greet Person Subtree")
    greet = Say(name="Greet", node=node, message="Hello!")
    name_wait_sel = py_trees.composites.Selector(name="Name Wait Sel")
    get_name_seq = py_trees.composites.Sequence(name="Get Name Seq")
    ask_for_name = Say(name="Ask For Name", node=node, message="Whats your name?")
    head_touched = IsHeadTouched(name="Head Touched", node=node)
    is_door_open = IsDoorOpen(name="Is Door Open", node=node)
    complete = Say(name="Complete", node=node, message="Complete")
    
    ask_name_again_seq = py_trees.composites.Sequence(name="Ask Name Again Seq")
    ask_name_again = Say(name="Ask Name Again", node=node, message="I'm sorry I didnt catch that")
    #reset_head_touched = ResetHeadTouched(name="Reset Head Touched", node=node)
    fail = py_trees.behaviours.Failure(name="Failure")
    
    name_retry = Standby(name="Name Retry", node=node)
    
    
    
    ask_name_again_seq.add_children([ask_name_again, fail])
    
    get_name_seq.add_children([ask_for_name, head_touched, is_door_open, complete])
    
    greet_person_subtree.add_children([greet, name_wait_sel])
    
    name_wait_sel.add_children([get_name_seq, ask_name_again_seq, name_retry])
    
    return greet_person_subtree