from launch import LaunchDescription
from launch.substitutions import LaunchConfiguration
from launch_ros.actions import Node
from nav2_common.launch import RewrittenYaml


def generate_launch_description():
    print("Localization launch")

    remappings = []  # syntax: from, to [("/tf", "tf")]

    map_yaml_file = LaunchConfiguration("map")
    amcl_config_file = LaunchConfiguration("amcl_config")
    amcl_params = RewrittenYaml(source_file=amcl_config_file, param_rewrites={}, convert_types=True)

    LaunchConfiguration("amcl_config")

    autostart_lm = True  # Auto start lifecycle manager
    lifecycle_nodes = ["map_server", "amcl"]

    return LaunchDescription([
        Node(
            package="nav2_map_server",
            executable="map_server",
            output="screen",
            remappings=remappings,
            parameters=[{"yaml_filename": map_yaml_file}]
        ),
        Node(
            package="nav2_amcl",
            executable="amcl",
            output="screen",
            remappings=remappings,
            parameters=[amcl_params]  # Makes amcl hang
        ),
        Node(package="nav2_lifecycle_manager",
             executable="lifecycle_manager",
             output="screen",
             parameters=[
                 {"node_names": lifecycle_nodes},
                 {"autostart": autostart_lm}]
        )
    ])
