import os

from ament_index_python.packages import get_package_share_directory
from launch import LaunchDescription
from launch.actions import IncludeLaunchDescription, DeclareLaunchArgument
from launch.launch_description_sources import PythonLaunchDescriptionSource
from launch.substitutions import LaunchConfiguration
from launch_ros.actions import Node


def generate_launch_description():
    motion_dir = get_package_share_directory("lhw_motion")
    path_to_navigation_launch = os.path.join(motion_dir, "navigation.launch.py")
    path_to_localization_launch = os.path.join(motion_dir, "localization.launch.py")

    map_arg_cmd = DeclareLaunchArgument("map", description="Full path to map yaml configuration file.", default_value=os.path.join(motion_dir, "maps/testmap.yaml"))
    map_arg = LaunchConfiguration("map", default=os.path.join(motion_dir, "maps/testmap.yaml"))

    amcl_arg_cmd = DeclareLaunchArgument("amcl", description="Full path to amcl yaml configuration file.", default_value=os.path.join(motion_dir, "config/amcl.yaml"))
    amcl_arg = LaunchConfiguration("amcl", default=os.path.join(motion_dir, "config/amcl.yaml"))

    costmap_arg_cmd = DeclareLaunchArgument("costmap", description="Full path to costmap yaml configuration file.",
                                         default_value=os.path.join(motion_dir, "config/costmap.yaml"))
    costmap_arg = LaunchConfiguration("costmap", default=os.path.join(motion_dir, "config/costmap.yaml"))

    bt_nav_arg_cmd = DeclareLaunchArgument("bt_navigator", description="Full path to costmap bt_navigator configuration file.",
                                            default_value=os.path.join(motion_dir, "config/bt_navigator.yaml"))
    bt_nav_arg = LaunchConfiguration("bt_navigator", default=os.path.join(motion_dir, "config/bt_navigator.yaml"))

    ld =  LaunchDescription([
        Node(
            package='lhw_motion',
            executable='motion_behaviours'
        ),
        IncludeLaunchDescription(PythonLaunchDescriptionSource(path_to_navigation_launch),
                                 launch_arguments={"map": map_arg, "costmap_config": costmap_arg,
                                                   "bt_navigator_config": bt_nav_arg}.items()),
        IncludeLaunchDescription(PythonLaunchDescriptionSource(path_to_localization_launch), launch_arguments={"amcl_config": amcl_arg}.items()),
    ])

    ld.add_action(map_arg_cmd)
    ld.add_action(amcl_arg_cmd)
    ld.add_action(costmap_arg_cmd)
    ld.add_action(bt_nav_arg_cmd)
    return ld
