from launch import LaunchDescription
from launch_ros.actions import Node
from launch.substitutions import LaunchConfiguration
from nav2_common.launch import RewrittenYaml


def generate_launch_description():

    remappings = []  # syntax: from, to [("/tf", "tf")]
    autostart_lm = True  # Auto start life cycle manager

    costmap_config_file = LaunchConfiguration("costmap_config")
    costmap_params = RewrittenYaml(source_file=costmap_config_file, param_rewrites={}, convert_types=True)

    bt_navigator_config_file = LaunchConfiguration("bt_navigator_config")
    bt_navigator_params = RewrittenYaml(source_file=bt_navigator_config_file, param_rewrites={}, convert_types=True)


    # Managed by the lifecycle manager to make sure nodes are initialized correctly before executing
    lifecycle_nodes = ["controller_server",
                       "planner_server",
                       "recoveries_server",
                       "bt_navigator",
                       "waypoint_follower"]


    return LaunchDescription([
        Node(
            package="nav2_controller",
            executable="controller_server",
            output="screen",
            remappings=remappings,
            parameters=[costmap_params]  # Handles local costmap
        ),
        Node(
            package="nav2_planner",
            executable="planner_server",
            output="screen",
            remappings=remappings,
            parameters=[costmap_params]  # Handles global costmap
        ),
        Node(
            package="nav2_recoveries",
            executable="recoveries_server",
            output="screen",
            remappings=remappings
        ),
        Node(
            package="nav2_waypoint_follower",
            executable="waypoint_follower",
            output="screen",
            remappings=remappings
        ),
        Node(
            package="nav2_bt_navigator",
            executable="bt_navigator",
            output="screen",
            remappings=remappings,
            parameters=[bt_navigator_params]
        ),
        Node(
            package="nav2_lifecycle_manager",
            executable="lifecycle_manager",
            output="screen",
            parameters=[{"node_names": lifecycle_nodes,
                        "autostart": autostart_lm}]
        )
    ])
