
from EventT2E import EventT2E


class NodeNlpEmotionCore():
    def __init__(self):
        self.CLASS_NAMES = ['angry', 'disgust', 'fear', 'happy', 'neutral', 'sad', 'surprise']


    def event_t2e_init_zero(self, obj):
        obj.angry = 0.0
        obj.disgust = 0.0
        obj.fear = 0.0
        obj.happy = 0.0
        obj.neutral = 0.0
        obj.sad = 0.0
        obj.surprised = 0.0
        return obj


    def event_t2e_normalize(self, obj):
        sum = 0
        sum += obj.angry     if obj.angry > 0 else 0
        sum += obj.disgust   if obj.disgust > 0 else 0
        sum += obj.fear      if obj.fear > 0 else 0
        sum += obj.happy     if obj.happy > 0 else 0
        sum += obj.neutral   if obj.neutral > 0 else 0
        sum += obj.sad       if obj.sad > 0 else 0
        sum += obj.surprised if obj.surprised > 0 else 0

        obj.angry = obj.angry / sum
        obj.disgust = obj.disgust / sum
        obj.fear = obj.fear / sum
        obj.happy = obj.happy / sum
        obj.neutral = obj.neutral / sum
        obj.sad = obj.sad / sum
        obj.surprised = obj.surprised / sum

        obj.angry =     obj.angry       if obj.angry > 0 else 0.0
        obj.disgust =   obj.disgust     if obj.disgust > 0 else 0.0
        obj.fear =      obj.fear        if obj.fear > 0 else 0.0
        obj.happy =     obj.happy       if obj.happy > 0 else 0.0
        obj.neutral =   obj.neutral     if obj.neutral > 0 else 0.0
        obj.sad =       obj.sad         if obj.sad > 0 else 0.0
        obj.surprised = obj.surprised   if obj.surprised > 0 else 0.0

        return obj


    def get_aggregated_emotion_event(self, key): 
        emotion_event_arr = self.input_dict[key]
        arr_length = len(emotion_event_arr)
        if arr_length == 0:
            return None
        elif arr_length == 1:
            return emotion_event_arr[0]

        msg_emotion = EventT2E()
        msg_emotion.source_id = emotion_event_arr[0].source_id  # This is always true
        msg_emotion.speaker_id = emotion_event_arr[0].speaker_id # This might change, however most likey not
        msg_emotion = self.event_t2e_init_zero(msg_emotion)
        for event in emotion_event_arr:
            msg_emotion.angry += event.angry
            msg_emotion.disgust += event.disgust
            msg_emotion.fear += event.fear
            msg_emotion.happy += event.happy
            msg_emotion.neutral += event.neutral
            msg_emotion.sad += event.sad
            msg_emotion.surprised += event.surprised
        return self.event_t2e_normalize(msg_emotion)

    def boost_node_1(self, event):
        event.angry = event.angry * 1.2
        event.fear = event.fear * 1.2
        event.happy = event.happy * 1.2
        event.sad = event.sad * 1.2
        event.neutral = event.neutral * 1.2
        return event

    def boost_node_2(self, event):
        event.neutral = event.neutral * 1.2
        return event

    def fire_emotion_event(self, input_dict):
        self.input_dict = input_dict

        msg_emotion = EventT2E()
        msg_emotion.source_id = 3
        msg_emotion.speaker_id = ''  # Integrate with vision id
        msg_emotion = self.event_t2e_init_zero(msg_emotion)

        t2e_event_1 = self.get_aggregated_emotion_event('t2e-1')
        t2e_event_2 = self.get_aggregated_emotion_event('t2e-2')

        # t2e_event_1 = self.boost_node_1(t2e_event_1)
        # t2e_event_2 = self.boost_node_2(t2e_event_2)
        
        t2e_event_arr = [t2e_event_1, t2e_event_2]
        t2e_event_arr = filter(lambda obj: obj is not None, t2e_event_arr)
        angry = []
        disgust = []
        fear = []
        happy = []
        neutral = []
        sad = []
        surprised = []
        for event in t2e_event_arr:
            angry.append(float(event.angry))
            disgust.append(float(event.disgust))
            fear.append(float(event.fear))
            happy.append(float(event.happy))
            neutral.append(float(event.neutral))
            sad.append(float(event.sad))
            surprised.append(float(event.surprised))

        def approach1(msg_emotion, angry, disgust, fear, happy, neutral, sad, surprised):
            msg_emotion.angry = max(angry)
            msg_emotion.disgust = max(disgust)
            msg_emotion.fear = max(fear)
            msg_emotion.happy = max(happy)
            msg_emotion.neutral = max(neutral)
            msg_emotion.sad = max(sad)
            msg_emotion.surprised = max(surprised)
            msg_emotion = self.event_t2e_normalize(msg_emotion)
            return msg_emotion

        def approach2(msg_emotion, angry, disgust, fear, happy, neutral, sad, surprised):
            def get_sum(arr):
                sum = 0
                valid_count = 0
                null_count = 0
                for v in arr:
                    if v < 0:
                        null_count += 1
                    else:
                        sum += v
                        valid_count += 1
                if valid_count <= 0:
                    return -1
                avg_val = sum / valid_count
                sum += avg_val * null_count
                return sum

            msg_emotion.angry = get_sum(angry)
            msg_emotion.disgust = get_sum(disgust)
            msg_emotion.fear = get_sum(fear)
            msg_emotion.happy = get_sum(happy)
            msg_emotion.neutral = get_sum(neutral)
            msg_emotion.sad = get_sum(sad)
            msg_emotion.surprised = get_sum(surprised)
            msg_emotion = self.event_t2e_normalize(msg_emotion)
            return msg_emotion

        msg_emotion = approach1(msg_emotion, angry, disgust, fear, happy, neutral, sad, surprised)
        # msg_emotion = approach2(msg_emotion, angry, disgust, fear, happy, neutral, sad, surprised)

        temp_lst = [
            msg_emotion.angry,
            msg_emotion.disgust,
            msg_emotion.fear,
            msg_emotion.happy,
            msg_emotion.neutral,
            msg_emotion.sad,
            msg_emotion.surprised
        ]
        dominant_emotion = self.CLASS_NAMES[temp_lst.index(max(temp_lst))]
        msg_emotion.dominant_emotion = dominant_emotion

        return msg_emotion

