
from tensorflow import keras

from keras.preprocessing.sequence import pad_sequences
import pickle
import numpy as np

import os

from EventT2E import EventT2E

class NodeText2Emotion2():
    def __init__(self):
        path_to_here = os.path.abspath("src")
        print(path_to_here)
        model = keras.models.load_model(path_to_here + '/nlp_t2e_model')
        with open(path_to_here + '/tokenizer.pickle', 'rb') as handle:
            tokenizer = pickle.load(handle)

        self.MAXLEN = 500
        self.CLASS_NAMES = ['happy', 'fear', 'angry', 'sad', 'neutral'] 

        self.model = model
        self.tokenizer = tokenizer


    def generateEventT2E(self, source_id, speaker_id, emotion_arr):
        dominant_emotion = self.CLASS_NAMES[np.argmax(emotion_arr)]
        msg_emotion = EventT2E()
        msg_emotion.source_id = source_id
        msg_emotion.speaker_id = speaker_id  # Integrate with vision id
        msg_emotion.angry = float(emotion_arr[2])
        msg_emotion.disgust = float(-1)
        msg_emotion.fear = float(emotion_arr[1])
        msg_emotion.happy = float(emotion_arr[0])
        msg_emotion.neutral = float(emotion_arr[4])
        msg_emotion.sad = float(emotion_arr[3])
        msg_emotion.surprised = float(-1)
        msg_emotion.dominant_emotion = dominant_emotion
        return msg_emotion

    def analyse_text(self, text):
        seq = self.tokenizer.texts_to_sequences([text])
        padded = pad_sequences(seq, maxlen=self.MAXLEN)
        prediction = self.model.predict(padded)
        # print('MODEL PREDICTION IS:  ', prediction)
        # print('THE MAX ARGUMENT IS:  ', np.argmax(prediction))
        return prediction

    def text_2_emotion(self, msg):
        emotion_arr = self.analyse_text(str(msg))
        msg_emotion = self.generateEventT2E(
            source_id=2,
            speaker_id='',
            emotion_arr=emotion_arr[0]  # SAMI HAVE A LOOK AT THIS /OSCAR :)
        )
        return msg_emotion

