"""
    GPT, Generative Pre-trained Transformer. Has a big and general language mode
    which allows for completing text. Is used to make robot
    talk about anything (although not always truthfully).
    Takes text as input.
"""

import rclpy
from rclpy.node import Node
from std_msgs.msg import String
from lhw_interfaces.msg import Response
from lhw_interfaces.msg import Parameter

import text2emotion as t2e
import json


class SentimentAnalysis(Node):
    def __init__(self):
        super().__init__('sentiment_analysis')
        
        # get the speech to text data from 'dialogflow_response' topic
        self.text_sub = self.create_subscription(
            Response,
            'dialogflow_response',
            self.text_2_emotion,
            10
        )
        
        # publish data to the topic 'emotion'. The data type for the response is 'Response'
        self.response_pub = self.create_publisher(Response, 'emotion', 10)
        
        # set up logger for feedback and debug
        self.log = self.get_logger()
        self.log.info(self.__class__.__name__ + " is loading..... ")

    def text_2_emotion(self, text_msg):
        text = str(text_msg.recognized_text)

        if text == " ":
            return

        self.get_logger().info(f'sentiment_analysis > text_2_emotion: {text}')
        raw_emotion = t2e.get_emotion(text)
        emotion = json.dumps(raw_emotion)

        msg = Response()
        msg.source = Response.SENTIMENT_ANALYSIS
        msg.response = emotion

        self.response_pub.publish(emotion)


def main(args=None):
    rclpy.init(args=args)
    sentiment_analysis_node = SentimentAnalysis()

    rclpy.spin(sentiment_analysis_node)

    sentiment_analysis_node.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()
