import rclpy
from rclpy.node import Node
from lhw_interfaces.msg import Response
from lhw_interfaces.msg import EventT2E
import text2emotion as t2e


class NodeText2Emotion(Node):
    def __init__(self):
        super().__init__('text_2_emotion')
        
        # get the speech to text data from 'dialogflow_response' topic
        self.create_subscription(
            Response,
            'dialogflow_response',
            self.text_2_emotion,
            10
        )
        
        # Publish data to the topic 'text_emotion'
        # The data type for the response is 'EventT2E'
        self.broadcast_emotion = self.create_publisher(EventT2E, 'text_emotion', 10)
        print('TEXT_2_EMOTION_NODE_1 IS UP AND RUNNING')

    def generateEventT2E(self, source_id, speaker_id, emotion_dict):
        emotions = ["Happy", "Angry", "Surprise", "Sad", "Fear"]
        emotion_sum = 0
        strongest_emotion_intensity = 0
        strongest_emotion = "Neutral"
        for e in emotions:
            e_intensity = emotion_dict[e]
            emotion_sum += e_intensity
            if e_intensity > strongest_emotion_intensity:
                strongest_emotion_intensity = e_intensity
                strongest_emotion = e

        msg_emotion = EventT2E()
        msg_emotion.source_id = source_id
        msg_emotion.speaker_id = speaker_id  # Integrate with vision id

        msg_emotion.angry = float(emotion_dict['Angry']) / emotion_sum
        msg_emotion.disgust = float(-1) / emotion_sum
        msg_emotion.fear = float(emotion_dict['Fear']) / emotion_sum
        msg_emotion.happy = float(emotion_dict['Happy'])
        msg_emotion.neutral = (1.0 - (emotion_sum / 5)) / emotion_sum  # This might work
        msg_emotion.sad = float(emotion_dict['Sad']) / emotion_sum
        msg_emotion.surprised = float(emotion_dict['Surprise']) / emotion_sum

        msg_emotion.dominant_emotion = strongest_emotion.lower()
        return msg_emotion

    def text_2_emotion(self, text_msg):
        print('-----------------------------------')
        print('text_2_emotion_1: event recieved from dialog flow')
        text = str(text_msg.recognized_text)

        if text == " ":
            return

        emotion_dict = t2e.get_emotion(text)
        print(emotion_dict)
        msg_emotion = self.generateEventT2E(
            source_id=Response.TEXT_2_EMOTION_1,
            speaker_id='',
            emotion_dict=emotion_dict
        )
        print('')
        print('-----------------------------------')
        print('text_2_emotion_1: broadcast emotion')
        print(msg_emotion)
        print('')
        print('')
        self.broadcast_emotion.publish(msg_emotion)


def main(args=None):
    rclpy.init(args=args)
    text_2_emotion_node = NodeText2Emotion()
    rclpy.spin(text_2_emotion_node)
    text_2_emotion_node.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()
