"""
    GPT, Generative Pre-trained Transformer. Has a big and general language mode
    which allows for completing text. Is used to make robot
    talk about anything (although not always truthfully).
    Takes text as input.
"""

import rclpy
from rclpy.node import Node
from std_msgs.msg import String
from lhw_interfaces.msg import Response
from lhw_interfaces.msg import Parameter
from lhw_interfaces.msg import EventT2E

#--------------------NEW ADDITION FOR SENTIMENT ANALYSIS---SAMI------------------------------
import tensorflow as tf
from tensorflow import keras

from keras.preprocessing.sequence import pad_sequences
import time 
import pickle
import numpy as np

import os

class NodeText2Emotion(Node):
    def __init__(self, model, tokenizer):
        super().__init__('text_2_emotion_2')

        self.MAXLEN = 500
        self.CLASS_NAMES = ['happy', 'fear', 'angry', 'sad', 'neutral'] 
        # get the speech to text data from 'dialogflow_response' topic
        self.create_subscription(
            Response,
            'dialogflow_response',
            self.text_cb,
            10
        )
        self.model = model
        self.tokenizer = tokenizer
        
        # Publish data to the topic 'text_emotion'
        # The data type for the response is 'EventT2E'
        self.broadcast_emotion = self.create_publisher(EventT2E, 'text_emotion', 10)
        print('TEXT_2_EMOTION_NODE_2 IS UP AND RUNNING')

    def generateEventT2E(self, source_id, speaker_id, emotion_arr):
        dominant_emotion = self.CLASS_NAMES[np.argmax(emotion_arr)]
        msg_emotion = EventT2E()
        msg_emotion.source_id = source_id
        msg_emotion.speaker_id = speaker_id  # Integrate with vision id
        msg_emotion.angry = float(emotion_arr[2])
        msg_emotion.disgust = float(-1)
        msg_emotion.fear = float(emotion_arr[1])
        msg_emotion.happy = float(emotion_arr[0])
        msg_emotion.neutral = float(emotion_arr[4])
        msg_emotion.sad = float(emotion_arr[3])
        msg_emotion.surprised = float(-1)
        msg_emotion.dominant_emotion = dominant_emotion
        return msg_emotion

    def analyse_text(self, text):
        seq = self.tokenizer.texts_to_sequences([text])
        padded = pad_sequences(seq, maxlen=self.MAXLEN)
        prediction = self.model.predict(padded)
        # print('MODEL PREDICTION IS:  ', prediction)
        # print('THE MAX ARGUMENT IS:  ', np.argmax(prediction))
        return prediction

    def text_cb(self, text_msg):
        """ text_cb.data has the text data """
        if text_msg.recognized_text != " ":
            #--------------CHECK THE CODE BELOW WITH EXPORT SIMON T-----------------------------
            # print(str(text_msg.recognized_text))
        
            emotion_arr = self.analyse_text(str(text_msg.recognized_text))
            msg_emotion = self.generateEventT2E(
                source_id=Response.TEXT_2_EMOTION_2,
                speaker_id='',
                emotion_arr=emotion_arr[0]  # SAMI HAVE A LOOK AT THIS /OSCAR :)
            )
            self.broadcast_emotion.publish(msg_emotion)


def main(args=None):
    path_to_here = os.getcwd() + '/src/lhw_nlp/lhw_nlp/'
    model = keras.models.load_model(path_to_here + 'nlp')
    with open(path_to_here + 'tokenizer.pickle', 'rb') as handle:
        tokenizer = pickle.load(handle)

    rclpy.init(args=args)
    text_2_emotion_node = NodeText2Emotion(model, tokenizer)
    rclpy.spin(text_2_emotion_node)
    text_2_emotion_node.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()
