"""
    GPT, Generative Pre-trained Transformer. Has a big and general language mode
    which allows for completing text. Is used to make robot
    talk about anything (although not always truthfully).
    Takes text as input.
"""

import rclpy
from rclpy.node import Node
from std_msgs.msg import String
from lhw_interfaces.msg import Audio
from lhw_interfaces.msg import Response
from lhw_interfaces.msg import Parameter


class ToneAnalysis(Node):
    def __init__(self):
        super().__init__('tone_analysis')
        # get data from 'microphone_buffer' topic
        self.buffer_sub = self.create_subscription(Audio, 'microphone_buffer', self.audio_cb, 10)
        # publish data to the topic 'emotion'. The data type for the response is 'Response'
        self.response_pub = self.create_publisher(Response, 'emotion', 10)
        self.log = self.get_logger()
        self.log.info(self.__class__.__name__ + " is loading..... ")

    def audio_cb(self, audio_msg):
        """ audio_msg.data has the sound/voice data """
        self.get_logger().info('audio callback!')
        if audio_msg.data != " ":
            msg = Response()
            msg.source = Response.TONE_ANALYSIS
            msg.response = "TODO: THIS SHOULD BE THE PREDICTED EMOTION"
            self.get_logger().info('publishing: "%s"' % msg.response)


def main(args=None):
    rclpy.init(args=args)
    tone_analysis_node = ToneAnalysis()

    rclpy.spin(tone_analysis_node)

    tone_analysis_node.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()
