"""
    Uses Wolfram services to query, for example, information about the world.
"""

import rclpy
from rclpy.node import Node
from std_msgs.msg import String
from lhw_interfaces.msg import Response
from lhw_interfaces.msg import Parameter


class WolframNLP(Node):
    def __init__(self):
        super().__init__('wolfram_nlp')
        self.log = self.get_logger()
        self.log.warn(self.__class__.__name__+" has not yet been implemented.")  # TODO
        self.text_sub = self.create_subscription(String, 'text', self.text_cb, 10)
        self.context_sub = self.create_subscription(String, 'context', self.context_cb, 10)
        self.response_pub = self.create_publisher(Response, 'response', 10)
        
        
    def text_cb(self, text_msg):
        self.log.error("Text callback in Wolfram has not yet been implemented.")  # TODO
        msg = Response()
        msg.source = Response.WOLFRAM
        msg.recognized_text = text_msg.data
        msg.response = "A sample response from Wolfram"
        msg.confidence = 0.1
        msg.intention = "Wolfram intention"
        msg.parameters.append(Parameter(key="name", value="Paul"))
        msg.parameters.append(Parameter(key="location", value="Kitchen"))
        self.response_pub.publish(msg)

    def context_cb(self, context_msg):
        self.log.error("Context callback in Wolfram has not yet been implemented.")  # TODO
        msg = Response()
        msg.source = Response.WOLFRAM
        msg.recognized_text = ""
        msg.response = "A sample response from Wolfram"
        msg.confidence = 0.1
        msg.intention = "Wolfram intention"
        msg.parameters.append(Parameter(key="name", value="Paul"))
        msg.parameters.append(Parameter(key="location", value="Kitchen"))
        self.response_pub.publish(msg)


def main(args=None):
    rclpy.init(args=args)
    wolfram_nlp_node = WolframNLP()

    rclpy.spin(wolfram_nlp_node)

    wolfram_nlp_node.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()


