#ifndef LHW_QI__CAMERA_COMPONENT_HPP_
#define LHW_QI__CAMERA_COMPONENT_HPP_

#include "lhw_qi/visibility_control.h"

#include <sensor_msgs/msg/image.hpp>
#include <rclcpp/rclcpp.hpp>

#include <qi/applicationsession.hpp>
#include <qi/anyobject.hpp>

namespace lhw_qi
{

  class Camera : public rclcpp::Node
  {
  public:
    LHW_QI_PUBLIC
    explicit Camera(const rclcpp::NodeOptions & options);
    ~Camera();
    void initiate(qi::SessionPtr session);
  private:
    qi::SessionPtr session_;
    qi::AnyObject video_service_;

    // Id of the camera subscriber.
    std::string subscriber_id_;

    rclcpp::Publisher<sensor_msgs::msg::Image>::SharedPtr pub_;
    OnSetParametersCallbackHandle::SharedPtr callback_handler_;
    
    rclcpp::TimerBase::SharedPtr request_image_timer_;
    rclcpp::TimerBase::SharedPtr publish_images_timer_;

    void request_image();
    void publish();

    rcl_interfaces::msg::SetParametersResult param_change_callback(const std::vector<rclcpp::Parameter> & parameters);
    void convert_alimage_to_message(qi::AnyValue& frame, sensor_msgs::msg::Image & msg);
    std::string convert_alcolorspace_to_ros_encoding(int colorspace);
    std::vector<qi::Future<qi::AnyValue>> image_future_vector_;

  };

} // namespace lhw_qi

#endif // !LHW_QI__CAMERA_COMPONENT_HPP_
