#include "lhw_qi/joint_states_component.hpp"

using namespace std::chrono_literals;

namespace lhw_qi
{

  JointStates::JointStates(const rclcpp::NodeOptions & options)
    : Node("joint_states")
  {
    RCLCPP_INFO(this->get_logger(), "Joint States component starting...");

    rclcpp::QoS custom_qos_profile(1);
    joint_states_pub_ = create_publisher<sensor_msgs::msg::JointState>("joint_states", custom_qos_profile);

    auto request_joint_states_callback =
      [this]() -> void
      {
	// RCLCPP_INFO(this->get_logger(), "Requesting joint states");
	// Send request for joint states and append future to vector
	try {
	  joint_states_future_vector_.push_back(motion_service_.async<std::vector<double> >("getAngles", "Body", true ));
	}
	catch (std::exception& e)
	  {
	    RCLCPP_WARN(this->get_logger(), "getAngles exception: %s", e.what());
	  }
      };

    auto publish_joint_states_callback =
      [this]() -> void
      {
	// RCLCPP_INFO(this->get_logger(), "Joint State Future Vector Size: %d", joint_states_future_vector_.size());
	// Publish all futures with values to ROS2 and remove them from vector
	auto it = joint_states_future_vector_.begin();
	while (it != joint_states_future_vector_.end())
	  {
	    auto joint_states_future = *it;
	    if (joint_states_future.hasValue()) {
	      // RCLCPP_INFO(this->get_logger(), "Got joint states");
	      std::vector<double> joint_angles = joint_states_future.value();
	      it = joint_states_future_vector_.erase(it);
	      joint_states_msg_.position.clear();
	      for(auto &joint_angle : joint_angles)
		{
		  joint_states_msg_.position.push_back(joint_angle);
		}
	      joint_states_pub_->publish(joint_states_msg_);
	    }
	    else {
	      ++it;
	    }
	  }
	// std::vector<double> joint_angles = motion_service_.call<std::vector<double> >("getAngles", "Body", true );
      };

    request_states_timer_ = this->create_wall_timer(100ms, request_joint_states_callback);

    publish_received_states_timer_ = this->create_wall_timer(10ms, publish_joint_states_callback);
  }

  void JointStates::initiate(qi::SessionPtr session)
  {
    session_ = session;
    motion_service_ = session_->service("ALMotion");

    joint_states_msg_.name = motion_service_.call<std::vector<std::string>>("getBodyNames", "Body");
  }

} // namespace lhw_qi

#include "rclcpp_components/register_node_macro.hpp"

// Register the component with class_loader.
// This acts as a sort of entry point, allowing the component to be discoverable when its library
// is being loaded into a running process.
RCLCPP_COMPONENTS_REGISTER_NODE(lhw_qi::JointStates)
