#include "lhw_qi/leds_component.hpp"

#define _USE_MATH_DEFINES
#include <cmath>
#include <string>
#include <iostream>

namespace lhw_qi
{

  Leds::Leds(const rclcpp::NodeOptions & options)
    : Node("leds", options)
  {

    auto eye_color_cb =
      [this](const typename std_msgs::msg::ColorRGBA::SharedPtr msg) -> void
      {
        RCUTILS_LOG_INFO("Event detected!");
        leds_service_.call<void>("fadeRGB", "FaceLeds", msg->r, msg->g, msg->b, 0.1);
      };

    auto eye_angle_cb =
      [this](const typename std_msgs::msg::Float64::SharedPtr msg) -> void
      {
        // TODO: Fill with
        std::cout << "/* message */" << '\n';

        leds_service_.call<void>("off", "AllLeds");

        //std::vector<std::string> ALLedsProxy::listLED(const std::string& arg1)

        //boost::shared_ptr<ALLedsProxy> leds = boost::shared_ptr<ALLedsProxy>(new ALLedsProxy(getParentBroker()));
        // Create a new group
        std::vector<std::string> names;
        names.push_back("FaceLedRight0");
        names.push_back("FaceLedRight1");
        names.push_back("FaceLedRight2");
        names.push_back("FaceLedLeft0");
        names.push_back("FaceLedLeft6");
        names.push_back("FaceLedLeft7");
        leds_service_.call<void>("createGroup","90deg", names);
        // Switch the new group on

        std::vector<std::string> names2;
        names2.push_back("FaceLedRight2");
        names2.push_back("FaceLedRight3");
        names2.push_back("FaceLedRight4");
        names2.push_back("FaceLedLeft6");
        names2.push_back("FaceLedLeft5");
        names2.push_back("FaceLedLeft4");
        leds_service_.call<void>("createGroup","180deg", names2);
        // Switch the new group on


        std::vector<std::string> names3;
        names3.push_back("FaceLedRight4");
        names3.push_back("FaceLedRight5");
        names3.push_back("FaceLedRight6");
        names3.push_back("FaceLedLeft2");
        names3.push_back("FaceLedLeft3");
        names3.push_back("FaceLedLeft4");
        leds_service_.call<void>("createGroup","270deg", names3);
        // Switch the new group on


        std::vector<std::string> names4;
        names4.push_back("FaceLedRight6");
        names4.push_back("FaceLedRight7");
        names4.push_back("FaceLedRight0");
        names4.push_back("FaceLedLeft2");
        names4.push_back("FaceLedLeft1");
        names4.push_back("FaceLedLeft0");
        leds_service_.call<void>("createGroup","360deg", names4);
        // Switch the new group on



        if(msg->data <= (3.14/2) ){
          leds_service_.call<void>("on", "90deg");
        }

        else if(msg->data <= (3.14) ){
          leds_service_.call<void>("on", "180deg");
        }

        else if(msg->data <= (3.14 * 1.5)){
          leds_service_.call<void>("on", "270deg");
        }

        else{
          leds_service_.call<void>("on", "360deg");
        }

      };

    auto ear_angle_cb =
      [this](const typename std_msgs::msg::Float64::SharedPtr msg) -> void
      {
        leds_service_.call<void>("on", msg->data*180.0/M_PI, 5.0, true);
      };

    auto ear_percentage_cb =
      [this](const typename std_msgs::msg::Float64::SharedPtr msg) -> void
      {
        // TODO: Fill with code
          leds_service_.call<void>("setIntensity", "EarLeds", msg->data);
      };

    auto state_cb =
      [this](const typename std_msgs::msg::Bool::SharedPtr msg) -> void
      {
        if (msg->data){
          leds_service_.call<void>("on","AllLeds");
        }else{
          leds_service_.call<void>("off","AllLeds");
        }
      };


    // Create subscribers
    eye_color_sub_ = create_subscription<std_msgs::msg::ColorRGBA>("eye_color", 10, eye_color_cb);
    eye_angle_sub_ = create_subscription<std_msgs::msg::Float64>("eye_angle", 10, eye_angle_cb);
    ear_angle_sub_ = create_subscription<std_msgs::msg::Float64>("ear_angle", 10, ear_angle_cb);
    ear_percentage_sub_ = create_subscription<std_msgs::msg::Float64>("ear_percentage", 10, ear_percentage_cb);
    state_sub_ = create_subscription<std_msgs::msg::Bool>("turn_on_off", 10, state_cb);

  }

  void Leds::initiate(qi::SessionPtr session){
    session_ = session;
    leds_service_ = session_->service("ALLeds");
  }

}  // namespace lhw_qi

#include "rclcpp_components/register_node_macro.hpp"

// Register the component with class_loader.
// This acts as a sort of entry point, allowing the component to be discoverable when its library
// is being loaded into a running process.
RCLCPP_COMPONENTS_REGISTER_NODE(lhw_qi::Leds)
