#include "lhw_qi/microphone_component.hpp"

#define _USE_MATH_DEFINES
#include <cmath>
#include <string>
#include <iostream>
#include <vector>
#include <fstream>
#include <iostream>
#include <iterator>
#include <qi/jsoncodec.hpp>

namespace lhw_qi
{

  Microphone::Microphone(const rclcpp::NodeOptions & options)
    : Node("sMicrophone", options)
  { 
    //pub_ = this->create_publisher<std_msgs::msg::Int16>("microphone_buffer", 1000);
    pub_ = this->create_publisher<lhw_interfaces::msg::Audio>("microphone_buffer", 1000);
    //std::ofstream myfile ("data3.txt", std::ios::app);
  }

  QI_REGISTER_MT_OBJECT(Microphone, init, processRemote)

  Microphone::~Microphone(void)
  {
    pub_.reset();
    listener_.call<qi::AnyValue>("unsubscribe", "sMicrophone");
  }

   void Microphone::init(void)
  {
    int front_mic_channel_id = 3;
    listener_.call<qi::AnyValue>("setClientPreferences",
			     "sMicrophone",
			     16000,
			     front_mic_channel_id, 
			     0);

    listener_.call<qi::AnyValue>("subscribe", "sMicrophone"); 
    listener_.call<void>("enableEnergyComputation");
  }
  
  void Microphone::initiate(qi::SessionPtr session, boost::shared_ptr<Microphone> microphone_ptr)
  {
    session_ = session;

    session_->waitForService("ALAudioDevice");
    
    listener_ = session_->service("ALAudioDevice");
 
    session_->registerService("sMicrophone", qi::AnyObject( microphone_ptr ) );

    
    qi::AnyObject microphone = session_ -> service("sMicrophone");
    
    microphone.call<qi::AnyValue>("init");
    
  }

  void Microphone::processRemote(
    const int & nbOfChannels, 
    const int & nbrOfSamplesByChannel, 
    const qi::AnyValue & timestamp, 
    const qi::AnyValue & buffer
    ) {
                
    std::pair<char*, size_t> charBuffer = buffer.unwrap().asRaw();
    lhw_interfaces::msg::Audio msg = lhw_interfaces::msg::Audio();
    //RCUTILS_LOG_INFO("length: %s", qi::encodeJSON(charBuffer.second).c_str());
    uint8_t* data  = (uint8_t*) charBuffer.first;

    // front_mic_energy < 900 to 1000 means silence, this could change depending on environment
    float front_mic_energy = listener_.call<float>("getFrontMicEnergy"); // Other solution
    RCUTILS_LOG_INFO("FRONT ENERGY CALL: %f\n", front_mic_energy); // TODO: Compare with calculation


    std::vector<uint8_t> vec(data, charBuffer.second + data);
    //std::vector<uint8_t> chunk_;
    //chunk_.insert(std::end(chunk_), std::begin(vec), std::end(vec));



    /*
    /// TODO: In case API call is too slow this needs to be fixed. Currently its not needed
    /// Computation of the RMS power of the signal delivered by
    /// each microphone on a 170ms buffer
    /// init RMS power to 0
    front_mic_energy = 0;
    qi::AnyValue buf_cpy = buffer;
    /// Calculation of the RMS power for the front mic
    for(int i = 0; i < nbrOfSamplesByChannel; i++)
    {
      front_mic_energy += (buf_cpy[nbrOfSamplesByChannel + i]).toFloat() * (buf_cpy[nbrOfSamplesByChannel + i]).toFloat();
      /// NEVER REACHES THIS
    }
    front_mic_energy /= (float)nbrOfSamplesByChannel;
    front_mic_energy = sqrtf(front_mic_energy);

    RCUTILS_LOG_INFO("FRONT ENERGY CALCULATION: %f\n", front_mic_energy);
    */

    //if (chunk_.size() > 0 * charBuffer.second){
		msg.data = vec;
    msg.rms = front_mic_energy;
		pub_->publish(msg);
		//RCUTILS_LOG_INFO("length: %s", qi::encodeJSON(chunk_.second).c_str());
		//chunk_.clear();
    //}

    /*
    try
    {
      std::vector<signed short> vec;
      signed short h {1};
      vec.push_back(h);
      //vec.push_back(0);
      //std::vector<signed short> vec(data, charBuffer.second + data);
      std::ostream_iterator<signed short> output_iterator(myfile, ", ");
      std::copy(vec.begin(), vec.end(), output_iterator);
    }
    catch(const std::exception& e)
    {
      std::cerr << e.what() << '\n';
    }
    */
    
    


    //std::pair<char*, size_t> charBuffer = buffer.unwrap().asRaw();
    
    //RCUTILS_LOG_INFO("buffer %s", qi::encodeJSON(qibuffer).c_str());
    //RCUTILS_LOG_INFO("timestamp %s", qi::encodeJSON(timestamp).c_str());
    //RCUTILS_LOG_INFO("buffer %s", charBuffer.second.c_str());
    
    //RCUTILS_LOG_INFO("length: %s", qi::encodeJSON(charBuffer.second).c_str());
    //msg.data = data;
    

    //RCUTILS_LOG_INFO("length1");
    
    
    
    
    /*msg.frames = charBuffer.second;
    msg.channels = nbOfChannels;
    msg.sample_rate = 48000;
    msg.is_bigendian = false;
    */
    //msg.layout = lhw_interfaces::msg::Audio::LAYOUT_NON_INTERLEAVED;
    //RCUTILS_LOG_INFO("length2");

    
      //RCUTILS_LOG_INFO("length3");
    //unsigned char* buffer = (unsigned char*)testdata;
    //typedef std::vector<unsigned char> bufferType;
    
    /*bufferType::size_type size = strlen((const char*)charBuffer.first);
    bufferType vec(charBuffer.first, size + charBuffer.first);
    msg.data = vec;
    //RCUTILS_LOG_INFO("buffer %s", vec.to_string().c_str());
    
    //msg.data = (unsigned char*) charBuffer.first;
    pub_->publish(msg);*/
    
  }

  
}  // namespace lhw_qi


#include "rclcpp_components/register_node_macro.hpp"

// Register the component with class_loader.
// This acts as a sort of entry point, allowing the component to be discoverable when its library
// is being loaded into a running process.

RCLCPP_COMPONENTS_REGISTER_NODE(lhw_qi::Microphone)
