#include "lhw_qi/motion_component.hpp"


#define _USE_MATH_DEFINES
#include <cmath>
//#include 
#include <string>
#include <iostream>
#include <vector>
#include <geometry_msgs/msg/twist.hpp>
#include <lhw_interfaces/msg/robot_action.hpp>

namespace lhw_qi
{
  Motion::Motion(const rclcpp::NodeOptions & options)
    : Node("motion", options)
  {

    auto cmd_vel_cb =
      [this](const typename geometry_msgs::msg::Twist::SharedPtr msg) -> void
      {
        RCUTILS_LOG_INFO("Event detected!");
        //x and y in m/s and z in rad/s
        motion_service_.call<void>("move", msg-> linear.x, msg-> linear.y, msg-> angular.z);
      };

    auto move_to_cb =
      [this](const typename geometry_msgs::msg::Twist::SharedPtr msg) -> void
      {
        RCUTILS_LOG_INFO("Event detected! MOVE TO");
        //x and y in m, z in rad
        motion_service_.call<void>("moveTo", msg-> linear.x, msg-> linear.y, msg-> angular.z);
        motion_service_.call<void>("waitUntilMoveIsFinished");
        //move_finished_pub.publish();
      };

    
    auto robot_action_cb = 
      [this](const typename lhw_interfaces::msg::RobotAction::SharedPtr msg) -> void
      {
        RCUTILS_LOG_INFO("Event detected!");

        /*
        Ask if its possible to use switch statements
        */
        /*
        switch(msg->type) {
          case msg->REST: 
            motion_service_.call<void>("rest");
            break;

          case msg->WAKEUP: 
            motion_service_.call<void>("wakeUp");

          case msg->CLOSE_HAND:
            motion_service_.call<void>("closeHand");
            break;

          case msg->OPEN_HAND: 
            motion_service_.call<void>("openHand");
            break;

          // Use this with caution since nothing will stop Pepper from crashing into things
          case msg->SAFETY_OFF:
            motion_service_.call<void>("setExternalCollisionProtectionEnabled", "Move", false);
            break;

          case msg->SAFETY_ON:
            motion_service_.call<void>("setExternalCollisionProtectionEnabled", "Move", true);
            break;

          case msg->CENTER_HEAD:
            motion_service_.call<void>("setStiffnesses", "Head", 1.0f);
            motion_service_.call<void>("angleInterpolation", "Head", 0.0f, 1.0f, true);
            break;

          case msg->TILT_HEAD_UP:
            motion_service_.call<void>("setStiffnesses", "HeadPitch", 1.0f);
            motion_service_.call<void>("angleInterpolation", "HeadPitch", -1.0f, 1.0f, true);
            break;

          // To be used while navigating
          case msg->TILT_HEAD_DOWN:
            motion_service_.call<void>("setStiffnesses", "HeadPitch", 1.0f);
            motion_service_.call<void>("angleInterpolation", "HeadPitch", 1.0f, 1.0f, true);
            break;

          // Rotates lower arm so palm is facing upwards
          case msg->PALM_OPEN:
            if (msg->right_arm) {
              motion_service_.call<void>("setStiffnesses", "RElbowYaw", 1.0f);
              motion_service_.call<void>("angleInterpolation", "RElbowYaw", 1.0f, 1.0f, true);
            }
            else {
              motion_service_.call<void>("setStiffnesses", "LElbowYaw", 1.0f);
              motion_service_.call<void>("angleInterpolation", "LElbowYaw", -1.0f, 1.0f, true);
            }
            break;

          // Rotates lower arm so palm is facing inwards towards robot
          case msg->PALM_CLOSE:
            if (msg->right_arm) {
              motion_service_.call<void>("setStiffnesses", "RElbowYaw", 1.0f);
              motion_service_.call<void>("angleInterpolation", "RElbowYaw", 0.0f, 1.0f, true);
            }
            else {
              motion_service_.call<void>("setStiffnesses", "LElbowYaw", 1.0f);
              motion_service_.call<void>("angleInterpolation", "LElbowYaw", 0.0f, 1.0f, true);
            }
            break;

          // Point right with right arm
          case msg->POINT_ABS_RIGHT:
            std::vector<float> position = {0.0f, -1.0f, 0.0f};
            tracker_service_.call<void>("pointAt", "RArm", position, 0, 0.3);
            break;
          
          // Point left with left arm
          case msg->POINT_ABS_LEFT:
            std::vector<float> position = {1.0f, 3.0f, 0.0f};
            tracker_service_.call<void>("pointAt", "LArm", position, 0, 0.3);
            break;

          // Point straight
          case msg->POINT_ABS_STRAIGHT:
            if (msg->right_arm) {
              // Right arm
              std::vector<float> position= {0.0f, -1.0f, 0.0f};
              tracker_service_.call<void>("pointAt", "RArm", position, 0, 0.3);
            }
            else {
              // Left arm
              std::vector<float> position = {0.0f, 1.0f, 0.0f};
              tracker_service_.call<void>("pointAt", "LArm", position, 0, 0.3);
            }
            break;

          // Point at general given location
          case msg->POINT_AT:
            if (msg->right_arm) {
              // Right arm
              tracker_service_.call<void>("pointAt", "RArm", msg->xyz, 0, 0.3);
            }
            else {
              // Left arm
              tracker_service_.call<void>("pointAt", "LArm", msg->xyz, 0, 0.3);
            }
            break;

          // TODO: Maybe unnecessary?
          case msg->BREATHING_ON:
            motion_service_.call<void>("setBreathEnabled", "Arms", true);
            break;

          case msg->IDLE_POSTURE_ON:
            motion_service_.call<void>("setIdlePostureEnabled", "Arms", true);
            break;

          // Turn off Pepper automatically resetting it's arms
          case msg->IDLE_POSTURE_OFF:
            motion_service_.call<void>("setIdlePostureEnabled", "Arms", false);
            break;

          default:
            RCUTILS_LOG_INFO("UNKNOWN STATE " + msg->type);
        }
        */
        
        if (msg->REST) {
          motion_service_.call<void>("rest");
        }
        else if (msg->type == msg->WAKEUP) {
          motion_service_.call<void>("wakeUp");
        }
        else if (msg->type == msg->CLOSE_HAND) {
          motion_service_.call<void>("closeHand", "RHand");
        }
        else if (msg->type == msg->OPEN_HAND) {
          motion_service_.call<void>("openHand", "RHand");
        }
        // Use this with caution since nothing will stop Pepper from crashing into things
        else if (msg->type == msg->SAFETY_OFF) {
          motion_service_.call<void>("setExternalCollisionProtectionEnabled", "Move", false);
        }
        else if (msg->type == msg->SAFETY_ON) {
          motion_service_.call<void>("setExternalCollisionProtectionEnabled", "Move", true);
        }
        // Center Pepper's head, making it look straight forward
        else if (msg->type == msg->CENTER_HEAD) {
          motion_service_.call<void>("setStiffnesses", "Head", 1.0f);
          motion_service_.call<void>("angleInterpolation", "Head", 0.0f, 1.0f, true);
        }
        else if (msg->type == msg->TILT_HEAD_UP) {
          RCUTILS_LOG_INFO("TILTING HEAD!");
          motion_service_.call<void>("setStiffnesses", "HeadPitch", 1.0f);
          motion_service_.call<void>("angleInterpolation", "HeadPitch", -1.0f, 1.0f, true);
        }
        // To be used while navigating
        else if (msg->type == msg->TILT_HEAD_DOWN) {
          motion_service_.call<void>("setStiffnesses", "HeadPitch", 1.0f);
          motion_service_.call<void>("angleInterpolation", "HeadPitch", 1.0f, 1.0f, true);
        }
        // Rotates lower arm so palm is facing upwards
        else if (msg->type == msg->PALM_OPEN) {
          if (msg->right_arm) {
            motion_service_.call<void>("setStiffnesses", "RElbowYaw", 1.0f);
            motion_service_.call<void>("angleInterpolation", "RElbowYaw", 1.0f, 1.0f, true);
          }
          else {
            motion_service_.call<void>("setStiffnesses", "LElbowYaw", 1.0f);
            motion_service_.call<void>("angleInterpolation", "LElbowYaw", -1.0f, 1.0f, true);
          }
        }
        // Rotates lower arm so palm is facing inwards towards robot
        else if (msg->type == msg->PALM_CLOSE) {
          if (msg->right_arm) {
            motion_service_.call<void>("setStiffnesses", "RElbowYaw", 1.0f);
            motion_service_.call<void>("angleInterpolation", "RElbowYaw", 0.0f, 1.0f, true);
          }
          else {
            motion_service_.call<void>("setStiffnesses", "LElbowYaw", 1.0f);
            motion_service_.call<void>("angleInterpolation", "LElbowYaw", 0.0f, 1.0f, true);
          }
        }
        // Point right with right arm
        else if (msg->POINT_ABS_RIGHT) {
          std::vector<float> position = {0.0f, -1.0f, 0.0f};
          tracker_service_.call<void>("pointAt", "RArm", position, 0, 0.3);
        }
        // Point left with left arm
        else if (msg->type == msg->POINT_ABS_LEFT) {
          std::vector<float> position = {1.0f, 3.0f, 0.0f};
          tracker_service_.call<void>("pointAt", "LArm", position, 0, 0.3);
        }
        // Point straight
        else if (msg->type == msg->POINT_ABS_STRAIGHT) {
          if (msg->right_arm == true) {
            // Right arm
            std::vector<float> position= {0.0f, -1.0f, 0.0f};
            tracker_service_.call<void>("pointAt", "RArm", position, 0, 0.3);
          }
          else {
            // Left arm
            std::vector<float> position = {0.0f, 1.0f, 0.0f};
            tracker_service_.call<void>("pointAt", "LArm", position, 0, 0.3);
          }
        }
        // Point at general given location
        else if (msg->type == msg->POINT_AT) {
          if (msg->right_arm == true) {
            // Right arm
            tracker_service_.call<void>("pointAt", "RArm", msg->xyz, 0, 0.3);
          }
          else {
            // Left arm
            tracker_service_.call<void>("pointAt", "LArm", msg->xyz, 0, 0.3);
          }
        }
        // TODO: Maybe unnecessary?
        else if (msg->type == msg->BREATHING_ON){           
          motion_service_.call<void>("setBreathEnabled", "Arms", true);
        }
        else if (msg->type == msg->IDLE_POSTURE_ON) {
          motion_service_.call<void>("setIdlePostureEnabled", "Arms", true);
        }
        // Turn off Pepper automatically resetting it's arms
        else if (msg->type == msg->IDLE_POSTURE_OFF) {
          motion_service_.call<void>("setIdlePostureEnabled", "Arms", false);
        }
        else {
          RCUTILS_LOG_INFO("UNKNOWN STATE " + msg->type);
        }
      };

    // Create subscribers
    move_sub_ = create_subscription<geometry_msgs::msg::Twist>("cmd_vel", 10, cmd_vel_cb);
    move_to_sub_ = create_subscription<geometry_msgs::msg::Twist>("move_to", 10, move_to_cb);
    robot_action_sub_ = create_subscription<lhw_interfaces::msg::RobotAction>("robot_action", 10, robot_action_cb);
  }

  void Motion::initiate(qi::SessionPtr session){
    session_ = session;
    motion_service_ = session_->service("ALMotion");
    tracker_service_ = session_->service("ALTracker");
  }


}  // namespace lhw_qi
#include "rclcpp_components/register_node_macro.hpp"

// This acts as a sort of entry point, allowing the component to be discoverable when its library
