#include "lhw_qi/touch_component.hpp"

#include <qi/jsoncodec.hpp>

#include <boost/function.hpp>
#include <boost/bind.hpp>


namespace lhw_qi
{

  // Create a Touch "component" that subclasses the generic rclcpp::Node base class.
  // Components get built into shared libraries and as such do not write their own main functions.
  // The process using the component's shared library will instantiate the class as a ROS node.
  Touch::Touch(const rclcpp::NodeOptions & options)
    : Node("touch", options)
  {
    // Create a publisher of "std_mgs/String" messages on the "bumper" topic.
    pub_ = create_publisher<std_msgs::msg::String>("bumper", 10);

  }

  void Touch::touch_cb(qi::AnyValue a) {
    RCUTILS_LOG_INFO("Touch detected!");
    RCUTILS_LOG_INFO("Data: %s", qi::encodeJSON(a).c_str());
    std_msgs::msg::String msg;
    msg.data = qi::encodeJSON(a);
    pub_->publish(msg);
  }


  void Touch::initiate(qi::SessionPtr session){
    session_ = session;
    qi::AnyObject almemory = session->service("ALMemory");
    touch_sub_ = almemory.call<qi::AnyObject>("subscriber", "RightBumperPressed");
    // MORE INFO AT http://doc.aldebaran.com/2-4/dev/libqi/api/cpp/type/anyobject.html
    touch_sub_.connect("signal", boost::function<void(qi::AnyValue)>(boost::bind(&Touch::touch_cb, this, boost::placeholders::_1)));
  }
  
}  // namespace lhw_qi

#include "rclcpp_components/register_node_macro.hpp"

// Register the component with class_loader.
// This acts as a sort of entry point, allowing the component to be discoverable when its library
// is being loaded into a running process.
RCLCPP_COMPONENTS_REGISTER_NODE(lhw_qi::Touch)
