#!/usr/bin/env python
# -*- coding: utf-8 -*-

import qi
import rospy
import threading
import time

from lhw_nlp.msg import DialogflowResponse
from websocket_server import WebsocketServer # source: https://github.com/Pithikos/python-websocket-server


class TabletNode(object):
    """
    Show in the tablet what the robot hears and says.
    """

    def __init__(self, session):
        super(TabletNode, self).__init__()

        self.tablet_service = session.service("ALTabletService")
        self.text_sub = rospy.Subscriber("dialogflow_response", DialogflowResponse, self.text_callback)
        self.connected = False


    """ Send text messages gotten from dialogflow to the tablet. """
    def text_callback(self, dialogflow_response):
        # Sends a message to all connected clients. The message is a simple string.
        self.server.send_message_to_all('said' + dialogflow_response.say_answer)
        self.server.send_message_to_all('heard' + dialogflow_response.recognition_result)

    """ Set callbacks, create threads, etc. """
    def connect_to_tablet(self, port):
        # A websocket server waiting for clients to connect: port(int) = port to bind to; host(str): ip to listen for connections.
        self.server = WebsocketServer(port, host='0.0.0.0')
        self.server.set_fn_new_client(self.new_client) # sets a callback function that will be called for every new client
        self.server.set_fn_client_left(self.client_left) # sets a callback function that will be called for every client disconnecting
        self.server.set_fn_message_received(self.message_received) # sets a callback function that will be called when a client sends a message

        # -------- fatta inget piss på hur det här nere funkar men aja
        wst = threading.Thread(target=self.server.run_forever)
        wst.daemon = True
        wst.start()

        conn_timeout = 30
        while not self.connected and conn_timeout:
            rospy.loginfo("Waiting for tablet to connect ...")
            time.sleep(1)
            conn_timeout -= 1
        rospy.loginfo("Connected to tablet")


    """ Callback for every new client connecting. """
    def new_client(self, client, server):
        rospy.loginfo("New client was connected and was given id {}".format(client['id'])) # obs: client is just a dictionary: {'id': client_id, 'handler': client_handler, 'address': (addr, port)}
        self.connected = True

    """ Callback called for every client disconnecting. """
    def client_left(self, client, server):
        rospy.loginfo("Client disconnected has id {}".format(client['id']))

    """ Called when a client sends a message. """
    def message_received(self, client, server, message):
        if len(message) > 200:
            message = message[:200] + "..."
        rospy.loginfo("Client ({}) said: {}".format(client['id'], message))

    def load_webpage(self):
        self.tablet_service.cleanWebview() # Clean the web browser.
        while True:
            # The ip of the robot from the tablet is 198.18.0.1
            if self.tablet_service.showWebview("http://198.18.0.1:80/apps/speechtotext/client.html"): # OBS: not sure if this local URL is correct
                break
            else:
                rospy.logerr("Couldn't load website on tablet.")
                time.sleep(3)
        rospy.loginfo("Showing website on tablet!")


def main():
    rospy.init_node("tablet_node", anonymous=True)

    if not rospy.has_param("~ip"):
        rospy.logerr("Missing parameter: ip ")
        return
    if not rospy.has_param("~port"):
        rospy.logerr("Missing parameter: port")
        return
    addr = "tcp://" + str(rospy.get_param("~ip")) + ":" + str(rospy.get_param("~port"))

    session = qi.Session()
    try:
        session.connect(addr)
    except RuntimeError:
        rospy.logerr("Can't connect to addr: " + addr)

    tablet_node = TabletNode(session)
    tablet_node.load_webpage()
    tablet_node.connect_to_tablet(8080)

if __name__ == "__main__":
    main()
