"""
    For debugging. Takes the local webcam and publishes it on a topic.
"""
import rclpy
from rclpy.node import Node
from sensor_msgs.msg import Image, CompressedImage
from sensor_msgs import msg
from cv_bridge import CvBridge, CvBridgeError

#from PIL import Image
import cv2
import numpy as np
from lhw_vision import VISION_ROOT

class DebugImagePublisher(Node):
    def __init__(self):
        super().__init__('DebugImagePublisher')
        self.log = self.get_logger()
        self.cv_bridge = CvBridge()

        self.image_publisher = self.create_publisher(msg.Image, "image", 10)
        self.image_publisher_compressed = self.create_publisher(msg.CompressedImage, "image_compressed", 10)
        self.timer = self.create_timer(0.5, self.timer_callback)

    def timer_callback(self):
        image = np.array(cv2.imread(f"{VISION_ROOT}/static/sofa.jpg"))
        image_msg = self.cv_bridge.cv2_to_imgmsg(image,encoding = "rgb8")
        image_msg.header.stamp = self.get_clock().now().to_msg()
        self.image_publisher.publish(image_msg)

        compressed_image_msg = self.cv_bridge.cv2_to_compressed_imgmsg(image)
        self.image_publisher_compressed.publish(compressed_image_msg)


def main(args=None):
    rclpy.init(args=args)
    debug_image_publisher = DebugImagePublisher()

    rclpy.spin(debug_image_publisher)

    debug_image_publisher.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()
