import numpy as np

def depth_to_xyz(bbox,depth,K):
    """ Calculate the x,y,z coordinate of the center of a bbox given a depthmap and camera intrinsics.
    First we inverse map image coords to normalized coordinates:
    (x',y',1) = K^-1 (u,v,1). 
    z = depth, and
    x = z*x'
    y = z*y'

    """
    h,w = depth.shape
    depth_box = depth[bbox[1]:bbox[3],bbox[0]:bbox[2]]
    median_depth = np.median(depth_box)
    
    u,v = (bbox[0]+bbox[2])//2,(bbox[1]+bbox[3])//2 # get the middle of the box
    v,u = min(max(0,v),h-1),min(max(0,u),w-1)
    
    coord = np.array([u,v,1])
    x_prim,y_prim,one = np.linalg.solve(K,coord)
    assert abs(one-1.0) < 1e-4,f"K is not correct"
    z = median_depth
    x = z*x_prim
    y = z*y_prim
    return (x,y,z)