import torch
import urllib.request
import cv2
import numpy as np
import matplotlib.pyplot as plt


class Midas:
    def __init__(self,a=0.0005,b=0.05):
        # TODO: https://github.com/intel-isl/MiDaS/issues/42 we need to accurately measure the distance to at least 2 points using peppers camera
        # Then we can resolve a,b
        self.device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
        self.midas = torch.hub.load("intel-isl/MiDaS", "MiDaS").to(self.device)
        self.transform = torch.hub.load("intel-isl/MiDaS", "transforms").default_transform
        self.a = a
        self.b = b

    def __call__(self,image):
        return self.get_depth(image)
    
    def get_depth(self,image):
        input_batch = self.transform(image).to(self.device)
        with torch.no_grad():
            self.midas.eval()
            prediction = self.midas(input_batch)
            prediction = torch.nn.functional.interpolate(
                prediction.unsqueeze(1),
                size=image.shape[:2],
                mode="bicubic",
                align_corners=False,
            ).squeeze()

        output = prediction.cpu().numpy()
        output = output*self.a+self.b
        output = 1/output
        return output