"""
    For debugging. Takes the local webcam and publishes it on a topic.
"""
import rclpy
from rclpy.node import Node
from sensor_msgs.msg import Image, CompressedImage

import cv2
from cv_bridge import CvBridge, CvBridgeError


class WebcamPublisher(Node):
    def __init__(self):
        super().__init__('webcam_publisher')
        self.camera = camera = cv2.VideoCapture(0)

        self.log = self.get_logger()
        self.cv_bridge = CvBridge()

        self.image_publisher = self.create_publisher(Image, "front_rgb_image", 10)
        self.image_publisher_compressed = self.create_publisher(CompressedImage, "front_rgb_image_compressed", 10)
        self.timer = self.create_timer(0.05, self.timer_callback)

    def timer_callback(self):
        return_value, image = self.camera.read()
        assert return_value != 0, "Got webcam source return value: "+str(return_value)

        # Publish
        image_msg = self.cv_bridge.cv2_to_imgmsg(image)
        self.image_publisher.publish(image_msg)

        compressed_image_msg = self.cv_bridge.cv2_to_compressed_imgmsg(image)
        self.image_publisher_compressed.publish(compressed_image_msg)


def main(args=None):
    rclpy.init(args=args)
    webcam_publisher_node = WebcamPublisher()

    rclpy.spin(webcam_publisher_node)

    webcam_publisher_node.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()
